
var JKarray = (function() {

	function JKarray(obj) {
		JKarray._super.constructor.apply(this);

		this.__p.type = 'array';
		this.__p.props = [];
		this.__p.attached = true;

		Object.defineProperty(this, 'length', {
			// Create a getter for the property
			enumerable: false,
			get: function () {
				return this.__p.props.length;
			},
			set: function(len) {
				var i;
				while ( this.__p.props.length < len) {
					this.push(null);
				}
				while ( this.__p.props.length > len) {
					this.pop();
				}
			}
		});
	}

	JK.inherit(JKarray, JKobject);
	
	JKarray.prototype.push = function(arrayElement) {

		var index = this.__p.props.length;
		var value = JK.model( arrayElement );
		this.__p.props.push( value );
		if (value instanceof JKobject) {
			value.setParent(this, index);
		}
		this.createGetSetProperty(index);

		if (this.__p.attached) {
			this.trigger( null, 'add'	, this, index	, '*'		, this.__p.props[index] );
			this.trigger( null, 'change', this, 'length', 'length'	, this.__p.props.length, this.__p.props.length - 1 );
		}

	};

	JKarray.prototype.__locationOf = function(element, start, end) {

		//if (this.__p.props.length === 0) return -1;

		start = start || 0;
		end = end || this.__p.props.length;
		var pivot = (start + end) >> 1;  // should be faster than the above calculation

		var c = this.__p.sortFunction(element, this.__p.props[pivot]);
		if (end - start <= 1) return c < 0 ? pivot : pivot + 1;
		
		if ( c < 0 ) return this.__locationOf(element, start, pivot);
		if ( c > 0 ) return this.__locationOf(element, pivot, end);

		return pivot;

	};

	JKarray.prototype.add = function(arrayElement) {
		
		if (!this.__p.sortFunction || !this.__p.props.length) {
			this.push(arrayElement);
		} else {
			// array length is > 0
			var index = this.__locationOf( arrayElement );
			// insert
			this.__p.props.splice(index, 0, JK.model(arrayElement) );

			// refresh parent prop. name of existing items (behind inserted items)
			var prop;
			for (var i = index, len = this.__p.props.length; i < len; i++) {
				prop = this.__p.props[i];
				if (prop instanceof JKobject) {
					prop.setParent(this, i);
				}
			}

			// object properties to fit array length
			this.createGetSetProperty(this.__p.props.length - 1);

			// trigger events
			if (this.__p.attached) {
				this.trigger( null, 'add'	, this, index	, '*'		, this.__p.props[index] );
				this.trigger( null, 'change', this, 'length', 'length'	, this.__p.props.length, this.__p.props.length - 1 );
			}
		}

	};

	JKarray.prototype.unshift = function(arrayElement) {

		var index = this.__p.props.length;
		var value = JK.model( arrayElement );
		this.__p.props.unshift( value );
		// refresh parent prop. name of all items
		for (var i = 0, len = this.__p.props.length; i < len; i++) {
			var prop = this.__p.props[i];
			if (prop instanceof JKobject) {
				prop.setParent(this, i);
			}
		}
		this.createGetSetProperty(index);

		if (this.__p.attached) {
			this.trigger( null, 'add'	, this, 0		, '*'		, this.__p.props[0] );
			this.trigger( null, 'change', this, 'length', 'length'	, this.__p.props.length - 1 );
		}
		
	};

	JKarray.prototype.pop = function() {

		var index = this.__p.props.length - 1;
		if (index >= 0) {
			delete this[index];
			var oldVal = this.__p.props.pop();
			if (this.__p.attached) {
				this.trigger( null, 'del'	, this, index	, '*'		, undefined				, oldVal 			);
				this.trigger( null, 'change', this, 'length', 'length'	, this.__p.props.length	, this.__p.props.length + 1 	);
			}
			if (oldVal instanceof JKobject) {
				oldVal.removeParent(this);
			}
			return oldVal;
		}

	};

	JKarray.prototype.shift = function() {

		if (this.__p.props.length > 0) {
			// delete the last property, because indexes are moving by -1
			delete this[this.__p.props.length - 1];
			var oldVal = this.__p.props.shift();
			// refresh parent prop. name of existing items
			for (var i = 0, len = this.__p.props.length; i < len; i++) {
				var prop = this.__p.props[i];
				if (prop instanceof JKobject) {
					prop.setParent(this, i);
				}
			}
			if (this.__p.attached) {
				this.trigger( null, 'del'	, this, 0		, '*'		, undefined		, oldVal			);
				this.trigger( null, 'change', this, 'length', 'length'	, this.__p.props.length	, this.__p.props.length + 1	);
			}
			if (oldVal instanceof JKobject) {
				oldVal.removeParent(this);
			}
			return oldVal;
		}

	};

	/**
	 * returns a standard JS array!!!
	 * 
	 * @param pos
	 * @param count
	 * @returns {Array}
	 */
	JKarray.prototype.splice = function(pos, count) {
		var i, len;
		var deleted = [];

		if (typeof count == 'undefined') count = this.__p.props.length - pos;
		count = Math.min(count, this.__p.props.length - pos);

		var recentLength = this.__p.props.length;
		var insertsLength = 0;

		i = pos;
		var max = Math.min(pos + count, this.__p.props.length);
		while (i < max) {
			deleted.push(this.__p.props[i]);
			i++;
		}

		var spliceArgs = [pos, count];
		for ( i = 2, len = arguments.length; i < len; i++) {
			spliceArgs.push( JK.model(arguments[i]) );
			insertsLength++;
		}
		Array.prototype.splice.apply( this.__p.props, spliceArgs );

		// refresh parent prop. name of existing items
		for (i = pos, len = this.__p.props.length; i < len; i++) {
			if (this.__p.props[i] instanceof JKobject) {
				this.__p.props[i].setParent(this, i);
			}
		}

		// delete or add object properties to fit array length
		for ( i = this.__p.props.length; i < recentLength; i++) {
			delete this[i];
		}
		for ( i = recentLength, len = this.__p.props.length; i < len; i++) {
			this.createGetSetProperty(i);
		}

		// trigger events
		for ( i = 0, len = deleted.length; i < len; i++) {
			if (this.__p.attached) {
				this.trigger( null, 'del', this, pos	, '*'		, undefined, deleted[i] );
			}
			if (deleted[i] instanceof JKobject) {
				deleted[i].removeParent(this);
			}
		}
		if (this.__p.attached) {
			for ( i = 0; i < insertsLength; i++) {
				this.trigger( null, 'add', this, pos + i, '*'		, this.__p.props[pos + i] );
			}

			this.trigger( null, 'change', this, 'length', 'length'	, this.__p.props.length, recentLength );
		}

		return deleted;

	};

	JKarray.prototype.indexOf = function(val, start) {
		return this.__p.props.indexOf(val, start);
	};

	JKarray.prototype.lastIndexOf = function(val, start) {
		return this.__p.props.lastIndexOf(val, start);
	};

	JKarray.prototype.slice = function(start, end) {

		var sl = this.__p.props.slice(start, end);
		return JK.model(sl);

	};

	JKarray.prototype.sort = function(sortFn) {
		var i, len;

		this.__p.sortFunction = sortFn || this.__p.sortFunction || function(a,b) { return (a > b) ? 1 : (a == b) ? 0 : -1; };

		var sortList = [];
		var reorderList = [];
		for (i = 0; i < this.__p.props.length; i++ ) {
			sortList.push(this.__p.props[i]);
		}

		this.__p.props.sort(this.__p.sortFunction);

		for (i = 0, len = this.__p.props.length; i < len; i++ ) {
			if (this.__p.props[i] instanceof JKobject) {
				this.__p.props[i].setParent(this, i);
			}
			reorderList.push(this.__p.props.indexOf(sortList[i]));
		}

		// trigger sort event
		if (this.__p.attached) {
			this.trigger( null, 'sort', this, undefined, '__this', reorderList );
		}

	};

	JKarray.prototype.sortBy = function(propPath, direction) {

		function getValue(obj, prop) {

			var path = prop.split('.');

			if (path.length > 1) {
				var key = path.shift();
				var subobj = obj[key];
				if ( (subobj) && (typeof subobj == 'object') ) {
					return getValue(subobj, path.join('.'));
				} else {
					return "";
				}

			} else {
				if ( typeof obj[path[0]] == 'function' ) {
					return obj[path[0]]();
				} else {
					return obj[path[0]];
				}
			}

		}

		if (direction == 'asc') direction = 1;
		if (direction == 'desc') direction = -1;
		var dir = direction || 1;

		this.__p.sortFunction = function (a, b) {
			var aVal = getValue(a, propPath);
			var bVal = getValue(b, propPath);
			if (typeof aVal == 'string') aVal = aVal.toLowerCase();
			if (typeof bVal == 'string') bVal = bVal.toLowerCase();

			return (aVal > bVal) ? dir : (aVal == bVal) ? 0 : -dir;
		};

		this.sort();

	};

	JKarray.prototype.reverse = function() {
		var i, len;
		var reorderList = [];

		this.__p.props.reverse();

		for (i = 0, len = this.__p.props.length; i < len; i++ ) {
			if (this.__p.props[i] instanceof JKobject) {
				this.__p.props[i].setParent(this, i);
			}
			reorderList.push(len - 1 - i);
		}

		// trigger sort event
		if (this.__p.attached) {
			this.trigger( null, 'sort', this, undefined, '__this', reorderList );
		}

	};

	JKarray.prototype.concat = function(arr) {
		var res = JK.model([]);
		res.extend(this);
		for (var i = 0, len = arr.length; i < len; i++) {
			res.push(arr[i]);
		}
		return res;
	};

	JKarray.prototype.join = function(seperator) {
		return this.__p.props.join(seperator);
	};

	JKarray.prototype.map = function(cb, scope) {
		return JK.model(this.__p.props.map(cb, scope));
	};

	JKarray.prototype.filter = function(cb, scope) {
		return JK.model(this.__p.props.filter(cb, scope));
	};

	JKarray.prototype.forEach = function(cb, scope) {
		return this.__p.props.forEach(cb, scope);
	};

	JKarray.prototype.every = function(cb, scope) {
		return this.__p.props.every(cb, scope);
	};

	JKarray.prototype.some = function(cb, scope) {
		return this.__p.props.some(cb, scope);
	};

	// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/find
	JKarray.prototype.find = function(cb) {

		if (Array.prototype.find) {
			return this.__p.props.find(cb);
		} else {
			if (typeof cb !== 'function') {
				throw new TypeError('callback must be a function');
			}

			var value;
			for (var i = 0, len = this.__p.props.length; i < len; i++) {
				value = this.__p.props[i];
				if (cb.call(this, value, i, this)) {
					return value;
				}
			}
			return undefined;
		}

	};

	// https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Array/findIndex
	JKarray.prototype.findIndex = function(cb) {

		if (Array.prototype.find) {
			return this.__p.props.findIndex(cb);
		} else {
			if (typeof cb !== 'function') {
				throw new TypeError('callback must be a function');
			}

			var value;
			for (var i = 0, len = this.__p.props.length; i < len; i++) {
				value = this.__p.props[i];
				if (cb.call(this, value, i, this)) {
					return i;
				}
			}
			return -1;
		}

	};
	
	JKarray.prototype.detach = function( inhibitEvents ) {
		this.__p.attached = !inhibitEvents;
		this.trigger( null, 'detach', this, undefined, '__this', this.__p.attached );		
	};
	
	JKarray.prototype.attach = function() {
		this.trigger( null, 'attach', this, undefined, '__this', this.__p.attached );		
		this.__p.attached = true;
	};
	
	JKarray.prototype.toJSON = function() {
		return this.__p.props;
	};
	

	return JKarray;

})();
