/**
 * inspired by Arjan van der Gaag
 * "The Javascript Class Pattern"
 * http://arjanvandergaag.nl/blog/javascript-class-pattern.html
 */
//function JKmixin(child, parent) {
//	for(prop in parent) {
//		if(parent.hasOwnProperty(prop)) {
//			child[prop] = parent[prop];
//		}
//	}
//}

var JKobject = (function() {

	function JKobject() {

		/*
		 * pseudo private members
		 */
		Object.defineProperty(this, '__p', {
			// make it invisible in own properties
			enumerable: false,
			// make it undeletable
			configurable: false,
			// Create a getter for the property
			value: {
				type: 'object',
				// array of { (ParentObject) obj, (String) prop }
				parents: [],
				abos: {
					change: {},
					add: {},
					del: {},
					sort: {},
					detach: {},
					attach: {},
				},
				cbArgs: [],
				props: {},
				sortFunction: null,
				circularId: 0,
				recursiveCircularId: 0,
				extendCircularId: 0,
			},
		});

	}

	JKobject.prototype.extendProp = function(prop, value, cid) {
		var ownValue;

		if ( this[prop] === undefined ) {
			// add new property
			if ((this.__p.type == 'array') && !isNaN(prop) ) {
				// this is an array and an array index
				prop = parseInt( prop );
				if (prop > this.__p.props.length) {
					for ( var i = this.__p.props.length; i < prop; i++) {
						this.push(null);
					}
				}
				this.push(value);

			} else {

				// this is an object

				this[prop] = ownValue = JK.model( value );
				// create a new property with getter/setter
				//this.createGetSetProperty(prop);

				if ( ownValue instanceof JKobject ) {
					// add object or array
//					ownValue.setParent(this, prop);
//					ownValue.recursiveTriggerAdd(null);
				} else if ( typeof value == 'function' ) {
					this.createFunctionEvents( prop );
				}

//				this.trigger( null, 'change', this, prop, prop, ownValue );

			}

		} else {
			ownValue = this[prop];
			if (
					( (ownValue instanceof JKobject) && JK.isObject(value) ) ||
					( (ownValue instanceof JKarray) && JK.isArray(value) ) 
			) {
				if (ownValue != value) {
					ownValue.extend(value, null, cid);
				}
			} else {
				// property exists
				value = JK.model(value);
				if (ownValue != value) {

					if (ownValue instanceof JKobject) {
						// remove parent reference of old value
						ownValue.removeParent( this );
					}
					this[prop] = value;
					if (value instanceof JKobject) {
						// add parent reference to new value
						value.setParent( this, prop );
					}

				}

			}
		}

	};

	JKobject.prototype.extend = function(obj, propBlacklist, cid) {

		cid = cid || JK.getUniqueId();
		if (cid == this.__p.extendCircularId) return;
		this.__p.extendCircularId = cid;

		var keys = Object.keys(obj),
			i = 0, 
			l = keys.length,
			prop;
		if (!propBlacklist) {
			for (; i < l; i++) {
				prop = keys[i];
				this.extendProp(prop, obj[prop], cid);
			}
		} else {
			for (; i < l; i++) {
				prop = keys[i];
				if (propBlacklist.indexOf( prop ) < 0 ) {
					this.extendProp(prop, obj[prop], cid);
				}
			}
		}
	};

	JKobject.prototype.init = function(data) {
		JKobject.prototype.extend.call(this, this);
		if (data) {
			this.extend(data);
		}

		if (this.__p.type == 'object') {
			Object.observe(this, function(changes){

				// This asynchronous callback runs
				changes.forEach(function(change) {
					// Letting us know what changed
					//console.log(change.type, change.name, change.oldValue);

					var prop = change.name;
					var val = change.object[prop];
//					if (change.type == 'add') {

					if ( (change.oldValue instanceof Date) && (val instanceof Date) ) {
						if (change.oldValue.getTime() != val.getTime()) {
							change.object.trigger( null, 'change', change.object, prop, prop, val, change.oldValue );
						}	
					} else {
						if (change.oldValue instanceof JKobject) {
							// remove parent reference of old value
							change.oldValue.removeParent( change.object );
						}
						if (val instanceof JKobject) {
							// add parent reference to new value
							val.setParent( change.object, prop );
							if (change.type == 'add') {
								val.recursiveTriggerAdd(null);
							}
						}

						change.object.trigger( null, 'change', change.object, prop, prop, val, change.oldValue );
					}
//					}
				});

			});
		}
	};

	JKobject.prototype.mixinProp = function(prop, value) {
		var i;

		if ( this[prop] !== undefined ) {
			
			// remove previous routes
			if ( this.constructor.prototype.routes ) {
				var changeRoutes = this.constructor.prototype.routes.change;
				for (var key in changeRoutes) {
					var changeRoute = changeRoutes[key];
					i = 0;
					while ( i < changeRoute.length) {
						if (changeRoute[i] == prop) {
							changeRoute.splice(i, 1);
						} else {
							i++; 
						}
					}
				}
			}			
			
		}
			
		// add the property

		this[prop] = JK.model( value );

		if ( typeof value == 'function' ) {

			var fn = this[prop].toString();
			var def = fn.match(/["']watch:\s*{{(.+?)}}["']/);
			if (def) {

				// find properties, that are used in this function
				// and bind a change event to this properties, that
				var matches = def[0].match(/{{(.+?)}}/g);
				if (matches) {
					// make sure, that the "routes" object exists 
					if (!this.constructor.prototype.routes) {
						this.constructor.prototype.routes = {change: {}};
					}
					var usedProp;
					for ( i = 0; i < matches.length; i++) {
						usedProp = matches[i].replace(/[}{]/g, '');

						// make sure, that the change-event array of "usedProp" exists
						if (!this.constructor.prototype.routes.change[usedProp]) {
							this.constructor.prototype.routes.change[usedProp] = [];
						}
						// prevent double entries
						if (this.constructor.prototype.routes.change[usedProp].indexOf(prop) == -1) {
							this.constructor.prototype.routes.change[usedProp].push(prop);
						}
					}

				}

			}

		}

	};

	JKobject.prototype.mixin = function(obj) {

		for ( var prop in obj ) {
			if (
					obj.hasOwnProperty( prop )
			) {
				this.mixinProp(prop, obj[prop]);
			}
		}

	};

	JKobject.prototype.createGetSetProperty = function(prop) {
		try {
			Object.defineProperty(this, prop, {
				// make it visible in own properties
				enumerable: true,
				// make it deletable
				configurable: true,
				// Create a getter for the property
				get: function () {
					return this.__p.props[prop];
				},
				// Create a setter for the property
				set: function (val) {
					var oldVal = this.__p.props[prop];
					this.__p.props[prop] = val;
					if ( (oldVal instanceof Date) && (val instanceof Date) ) {
						if (oldVal.getTime() != val.getTime()) {
							this.trigger( null, 'change', this, prop, prop, val, oldVal );
						}	
					} else if (val !== oldVal) {
						
						if (oldVal instanceof JKobject) {
							// remove parent reference of old value
							oldVal.removeParent( this );
						}
						if (val instanceof JKobject) {
							// add parent reference to new value
							val.setParent( this, prop );
						}

						this.trigger( null, 'change', this, prop, prop, val, oldVal );
					}
				}
			});
		} catch (e) {
			if (console) {
				console.log('prop: ' + prop + '; ' + this.constructor.name);
				console.log(e);
			}

		}

	};
	
	JKobject.prototype.watchCallback = function(e, val, obj, objProp, oldVal, prop) {
		obj.trigger( null, 'change', obj, prop, prop, obj[prop]() );
	};

	/**
	 * if the function returns a value,
	 * create change event, that triggers, when properties, which may change the function result, changed
	 *
	 * @param {String}		prop	property (function-)name
	 */
	JKobject.prototype.createFunctionEvents = function(prop) {

		var fn = this[prop].toString();
		var def = fn.match(/["']watch:\s*{{(.+?)}}["']/);
		if (def) {

			// find properties, that are used in this function
			// and bind a change event to this properties, that
			var matches = def[0].match(/{{(.+?)}}/g);
			if (matches) {
				var usedProp;
				for (var i = 0; i < matches.length; i++) {
					usedProp = matches[i].replace(/[}{]/g, '');

					this.on('change', usedProp, this.watchCallback, prop);
				}

			}

		}

	};

	/**
	 * trigger add events for all properties
	 *
	 * @param {Number}	id			triggerid, prevents circular triggering
	 */
	JKobject.prototype.recursiveTriggerAdd = function(id) {

		id = id || JK.getUniqueId();
		if (id == this.__p.recursiveCircularId) return;
		this.__p.recursiveCircularId = id;

		var //props = this.__p.props,
			keys = Object.keys(this),
			i = 0, 
			l = keys.length,
			isArray = this.isArray(),
			prop,
			path,
			value;
		for (; i < l; i++) {
			prop = keys[i];
			path = isArray ? '*' : prop;
			value = this[prop];
			this.trigger( null, 'change', this, prop, path, value );

			if ( value instanceof JKobject) {
				value.recursiveTriggerAdd(id);
			}
		}

	};
	
	var triggerQueue = [];
	var triggerQueueIDs = [];
	var triggerTimeoutID;
	function addToTriggerQueue( thisVar, cbId, cb, args ) {
		
		// find existing callback
		var newTrigger = true;
		var event = args[0];
		if (!thisVar.__p.__id) {
			thisVar.__p.__id = JK.getUniqueId();
		}
//		if (!cb.__id) {
//			cb.__id = JK.getUniqueId();
//		}
		var id = event + '-' + thisVar.__p.__id + '-' + cbId;
		if (event == 'change' && !thisVar.isArray()) {
			var i = triggerQueueIDs.indexOf( id );
			if ( i >= 0 ) {
				newTrigger = false;
				triggerQueue[i].args[1] = args[1];
			}
		}

		if (newTrigger) {
			triggerQueue.push({
				thisVar: thisVar,
				cb: cb,
				args: args
			});
			triggerQueueIDs.push(id);
		}

		if (!triggerTimeoutID) {
			triggerTimeoutID = setImmediate(function() {
				
				while (triggerQueue.length) {
					var triggerObj = triggerQueue.shift();
					triggerQueueIDs.shift();
					if (
							(typeof triggerObj.args[1] == 'function') &&
							(triggerObj.args[1].delayedMixin)
					) {
						// evaluate delayed mixin function result
						triggerObj.args[1] = triggerObj.args[1].call(triggerObj.thisVar);
					}
					triggerObj.cb.apply(triggerObj.thisVar, triggerObj.args);
				}
				triggerTimeoutID = null;
				
			});			
		}
	}

	/**
	 * trigger event
	 *
	 * @param {Number}	id				triggerid, prevents circular triggering
	 * @param {String}	event			event name
	 * @param {JKobject}	obj				affected object
	 * @param {Number}	prop			Property (index) of (array) element
	 * @param {String}	propPath		event-property or path
	 * @param {String}	origPropPath	event-property or path
	 * @param 			val				new (array) element
	 * @param 			oldVal			old (array) element
	 */
	JKobject.prototype.trigger = function(id, event, obj, prop, propPath, val, oldVal) {
		var i = 0, 
			len = 0;

		id = id || JK.getUniqueId();
		if (id == this.__p.circularId) return;
		this.__p.circularId = id;

		var abos = this.__p.abos[event][propPath];
		if (abos) {
			for (i = 0, len = abos.length; i < len; i++) {
				var cbId = abos[i] - 1;
				var cbArgs = this.__p.cbArgs[cbId];
				var cb = cbArgs[2];
				var args = [event, val, obj, prop, oldVal].concat(Array.prototype.slice.call(cbArgs, 3));
				try {
					//addToTriggerQueue(this, cbId, cb, args);
					if (
							(typeof val == 'function') &&
							(val.delayedMixin)
					) {
						// evaluate delayed mixin function result
						val = val.call(this);
					}
					cb.apply(this, args);
				} catch (e) {
					if (window.QUnit) {
						throw e;//.message;
					}
					if (console) {
						console.log('trigger('+event+', obj['+prop+'], path:'+propPath+', val:'+val+', old val:'+oldVal+') FAILED!');
						console.log(e);
					}
				}
			}
		}
		
		var routes = this.constructor.prototype.routes;
		if (
			routes &&
			routes[event] &&
			routes[event][propPath]
		) {
			var route = routes[event][propPath];
			for (i = 0, len = route.length; i < len; i++) {
				var routePath = route[i];
				var value = this[routePath]();
				if (typeof value == 'function') {
					value.delayedMixin = true;
				}
				this.trigger( null, 'change', this, routePath, routePath, value );
			}
		}

		// path trigger
		var parents = this.__p.parents;
		for (i = 0, len = parents.length; i < len; i++) {
			var parent = parents[i];
			var path;
			if (parent.obj.isArray()) {
				path	= ('*.' + propPath).replace(/[.]*__this$/, '');
				parent.obj.trigger(id, event, obj, prop, path, val, oldVal);
			} else {
				path	= (parent.prop + '.' + propPath).replace(/[.]*__this$/, '');
				parent.obj.trigger(id, event, obj, prop, path, val, oldVal);
			}
		}

	};

	JKobject.prototype.on = function(event, props, callback, data) {

		var p = this.__p,
			evt = p.abos[event],
			cbId;
		
		// test for type 'string'
		if (props.substring) {
			cbId = p.cbArgs.push( arguments );
			// one property
			if (!evt[props]) {
				evt[props] = [ cbId ];
			} else {
				evt[props].push(cbId);
			}
			
		} else {
			
			// multiple properties
			// TODO write a test
//			for( var i = 0; i < props.length; i++) {
//				var prop = props[i];
//
//				if (!evt[prop]) {
//					evt[prop] = [ callback ];
//				} else {
//					evt[prop].push(callback);
//				}
//
//			}

		}
		
		return cbId;

	};

	JKobject.prototype.off = function(cbId) {

		var event, propPath, i;

		var pCbArgs = this.__p.cbArgs;
		if (pCbArgs[cbId - 1]) {
			var cbArgs = pCbArgs[cbId - 1];
			event = cbArgs[0];
			propPath = cbArgs[1];
			i = this.__p.abos[event][propPath].indexOf(cbId);
			if (i >= 0) {
				this.__p.abos[event][propPath].splice(i, 1);
			}
			pCbArgs[cbId - 1] = null;
		}

		return this;

	};

	JKobject.prototype.setParent = function(parentModel, prop) {
		var parents = this.__p.parents;
		for (var i = 0; i < parents.length; i++) {
			if (parents[i].obj == parentModel) {
				parents[i].prop = prop;
				return;
			}
		}
		parents.push({
			obj: parentModel,
			prop: prop
		});
	};

	/**
	 * Destructor
	 * 
	 * @param {JKobject}	parentModel
	 */
	JKobject.prototype.removeParent = function(parentModel) {
		var i = 0;
		while ( i < this.__p.parents.length) {
			if (this.__p.parents[i].obj == parentModel) {
				this.__p.parents.splice(i, 1);
			} else {
				i++;
			}
		}
		if ( this.__p.parents.length === 0 ) {
			this.destruct();
		}
	};

	JKobject.prototype.destruct = function() {};

	JKobject.prototype.isArray = function() {
		return this.__p.type == 'array';
	};

	JKobject.prototype.toString = function() {
		return this.__p.props.toString();
	};

	return JKobject;

})();

//var JKconvertObject = (function() {
//    JKinherit(JKconvertObject, JKobject);
//    function JKconvertObject(obj) {
//    	JKconvertObject._super.constructor.apply(this);
//
//    	if (obj){
//    		this.extend(obj);
//    	}
//    }
//    return JKconvertObject;
//})();

