
var TmaConfig = (function() {

	function TmaConfig() {

		// call super class constructor
		TmaConfig._super.constructor.apply(this);

		// empty default status
		this.content			= "config";
		this.timestamp			= "";
		this.sysid				= 0;
		this.device				= '';
		this.apptype			= '';
		this.systemtype			= 0;
		this.bigendian			= 0;
		this.remotetype			= '';
		this.maxrecstreams		= 0;
		this.version			= 0;
		this.recextension		= '';
		this.language_code		= '';
		this.utftoppy			= 0;
		this.tma1ver			= 0;
		/**
		 * current WebControl version as number
		 */
		this.webControlVersion	= 0;
		this.blockfactor		= 0;
		this.autoppy			= 0;
		this.analogueinput		= 0;
		this.utcoffset			= 0;
		this.dst				= 0;
		this.pre_padding		= 0;
		this.post_padding		= 0;
		this.hddmodel			= '';
		this.hddserial			= '';
		this.hddfirmware		= '';
		this.pvrname			= '';
		this.logginglevel		= 0;
		this.epgsource			= '';
		this.module_count		= 0;
		/**
		 * This is the number of milliseconds difference between the local time and the PVR time.
		 */
		this.timeOffset 		= 0;
		this.debug				= false;
		this.mock				= false;

		// init
		this.init();

	}

	JK.inherit(TmaConfig, JKobject);

	/**
	 * Get the status object from the PVR
	 * @returns jQueryDeferred
	 */
	TmaConfig.prototype.load = function() {

		var self = this;
		var params = {
				'function':	'config',
				action:		'get!'
		};

		return $.getJSON('/api', params).done(function(data) {

			if ( data.modules ) {
				$.each( data.modules, function(i, module) {
					tma.fn.addModule(module.name, module.name_hex);
				});
			}

			// Calculate the number of milliseconds difference between the local time and the PVR time.
			var pvrNow = new Date(data.timestamp);
			var clientNow = new Date();
			self.timeOffset = clientNow - pvrNow;

			// get the TMA1 version as a number
			self.webControlVersion = parseInt( data.tma1ver.replace(/\D+/g, '') );

			delete data.modules;

			self.extend(data);

			// prepare status recordings
			for ( var i = 0; i < data.maxrecstreams; i++) {
				tma.status.recording.push( new TmaStatusRecording() );
			}

		}).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in TmaConfig.load()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
				}
			});

		});

	};

	return TmaConfig;

})();
