/**
 * encapsulates file object of json response
 *
 * @param {TmaFile}	file
 */
var TmaFile = (function() {

	function TmaFile( file ) {

		// call super class constructor
		TmaFile._super.constructor.apply(this);

		// empty default data
		this.name		= "";
		this.name_hex	= "";

		this.sortdate	= "";
		this.isPlaying	= false;
		/** @returns {TmaStatusRecording} */
		this.recording	= null;
		this.isRunningTap = false;

		// result of mixin updateName
		this.filename	= '';

		this.init( file );

		// static data => not observable
		this.timestamp	= "";
		this.size		= 0;
		this.inode		= 0;
		this.attr		= 0;
		/**
		 * file mode (permission bits)
		 */
		this.info		= 0;
		this.infpresent	= 0;
		this.tappresent	= 0;
		this.infdata	= this.infdata || null;
		this.tap		= this.tap || null;

	}
	JK.inherit(TmaFile, JKobject);

	TmaFile.prototype.extend = function(file) {

		// generate a value to sort for date
		if ( (file.attr == 208) && (file.infdata) && (file.infdata.header.starttime)) {
			file.sortdate = file.infdata.header.starttime;
		} else {
			file.sortdate = file.timestamp ;
		}

		// call inherited extend; but ignore infdata and tap
		this.constructor._super.extend.call(this, file, ['infdata', 'tap']);

		// insert non-observable data
		if (file.infdata) {
			this.infdata = file.infdata;
			this.infdata.exteventinfo.htmlText = function() {
				return this.text.replace(/(?:\r\n|\r|\n|\x8a)/g, '<br>');
			};
		}
		if (file.tap) {
			this.tap = file.tap;
		}

	};


	TmaFile.prototype.set = function(file) {

		this.name		= file.name;
		this.name_hex	= file.name_hex;
		this.timestamp	= file.timestamp;
		this.size		= file.size;
		this.inode		= file.inode;
		this.attr		= file.attr;
		/**
		 * file mode (permission bits)
		 */
		this.info		= file.info;
		this.infpresent	= file.infpresent;
		this.tappresent	= file.tappresent;
		// generate a value to sort for date
		if ( (file.attr == 208) && (file.infdata) && (file.infdata.header.starttime)) {
			this.sortdate = file.infdata.header.starttime;
		} else {
			this.sortdate = file.timestamp ;
		}

		if (file.infdata) {
			this.infdata = file.infdata;
			this.infdata.exteventinfo.htmlText = function() {
				return this.text.replace(/(?:\r\n|\r|\n|\x8a)/g, '<br>');
			};
		}
		if (file.tap) {
			this.tap = file.tap;
		}

	};

	TmaFile.prototype.isPlayable = function() {
		return !!(
				this.infpresent ||
				this.name.match(/\.rec$|\.mpg$|\.mkv$|\.ts$|\.vob$|\.m2ts$|\.divx$/)
		);
	};

	TmaFile.prototype.isTextfile = function() {
		return (
				!!this.name.match(/\.txt$|\.log|\.prn$|\.csv$|\.json$|\.ini$|\.sh$|\.lng$/)
		);
	};

	TmaFile.prototype.isFile = function() {
		return this.attr === 208;
	};

	/** @returns {TmaChannel} */
	TmaFile.prototype.channel = function() {
		if (this.infdata) {
			var channel = tma.channels.channel(this.infdata.header.logoid);
			if (!channel) {
				channel = new TmaChannel();
				channel.chtype = this.infdata.service.servicetype;
				channel.servicename = this.infdata.service.servicename;
			}
			return channel;
		}
		return null;
	};

	TmaFile.mixin({

		updateName: function() {
			"watch: {{name}}, {{name_hex}}";

			/** @returns {String} */
			var name = (tma.config.utftoppy || !this.name_hex) ?
					this.name || ''
					: tma.decodeHexname(this.name_hex);

			if (name != this.filename) {

				this.filename = name;

				// set isRunningTap
				var isRunning = false;
				var path = tma.files.directory + '/' + this.filename;
				for (var j = 0, jMax = tma.taps.taps.length; j < jMax; j++ ) {
					var tap = tma.taps.taps[j];
					if (tap.file_name == path) {
						isRunning = true;
						break;
					}
				}
				this.isRunningTap = isRunning;

				// set isPlaying
				path = tma.files.directory + '/' + this.filename + '.inf';
				this.isPlaying = (path == tma.status.absolutefilename);

				// set recording
				if (
//						this.infpresent &&
						(tma.files.directory == '/mnt/hd/DataFiles')
				) {

					for( var i = 0, iMax = tma.status.recording.length; i < iMax; i++ ) {
						var recording = tma.status.recording[i];
						if (name == recording.getFilename()) {
							this.recording = recording;
							break;
						}
					}

				}

			}

		},

		/**
		 * file name of this file without extension (non-UTF Toppy safe)
		 *
		 * @returns {String}
		 */
		filenameNoExt: function() {
			"watch: {{filename}}";

			return this.filename.replace(/\.[^.]+$/, '');

		},

		filenameFmt: function() {
			"watch: {{filename}}";
			return function() {
				if ( this.tappresent ) {
					return this.tap.name;
				} else {
					var removeExt = ( this.infpresent || this.tappresent );
					return (removeExt)? this.filenameNoExt() : this.filename;
				}
			};
		},

		filesizeFmt: function() {
			if (this.attr == 208) {
				if (		this.size /       1024 /*1KB*/ < 0.1 ) {
					return ( this.size ).toLocaleString() + ' B';
				} else if (	this.size /    1048576 /*1MB*/ < 0.1 ) {
					return (parseInt(this.size * 10 /       1024 /*1KB*/) / 10 ).toLocaleString() + ' KB';
				} else if (	this.size / 1073741824 /*1GB*/ < 0.1 ) {
					return (parseInt(this.size * 10 /    1048576 /*1MB*/) / 10 ).toLocaleString() + ' MB';
				} else {
					return (parseInt(this.size * 10 / 1073741824 /*1GB*/) / 10 ).toLocaleString() + ' GB';
				}
			} else {
				return '';
			}
		},

		durationFmt: function() {
			if (this.infdata) {
				return tma.formatDuration(this.infdata.header.durationmin);
			} else if (this.tap) {
				return this.tap.version;
			} else {
				return '';
			}
		},

	});

	return TmaFile;

})();

function TmaInfData() {

	// empty default status
	this.header			= new TmaInfHeader();
	this.service		= new TmaInfService();
	this.eventinfo		= new TmaInfEvent();
	this.exteventinfo	= new TmaInfExtEvent();
	this.tpinfo			= new TmaTpInfo();
	this.bookmarks		= new TmaInfBookmarks();

}

function TmaInfHeader() {

	var self = this;

	// empty default status
	this.starttime		= '';
	this.durationmin	= 0;
	this.durationsec	= 0;
	this.svcnum			= 0;
	this.svctype		= 0;
	this.logoid			= '';
	this.cryptflag		= 0;
	this.headerflags	= 0;
	this.headerflags2	= 0;
	this.flagtimeshift	= 0;
	this.flagcopy		= 0;

	/** @returns {TmaChannel} */
	this.channel = function() {
		return tma.channel(self.logoid);
	};

}

function TmaInfService() {

	// empty default status
	this.satindex		= 0;
	this.servicetype	= 0;
	this.tpindex		= 0;
	this.tunernum		= 0;
	this.flagskip		= 0;
	this.flaglock		= 0;
	this.flagcas		= 0;
	this.flagdel		= 0;
	this.serviceid		= 0;
	this.pmtpid			= 0;
	this.pcrpid			= 0;
	this.videopid		= 0;
	this.audiopid		= 0;
	this.servicename	= '';
	this.videostreamtype= 0;
	this.audiostreamtype= 0;

}

function TmaInfEvent() {

	// empty default status
	this.duration		= 0;
	this.eventid		= 0;
	this.starttime_utc	= '';
	this.starttime_local= '';
	this.endtime_utc	= '';
	this.endtime_local	= '';
	this.runningstatus	= 0;
	this.parental		= 0;
	this.parental_text	= '';
	this.serviceid		= 0;
	this.name			= '';
	this.description	= '';

}

function TmaInfExtEvent() {

	var self = this;

	// empty default status
	this.eventid		= 0;
	this.serviceid		= 0;
	this.text			= '';

	/**
	 * extended event text, where newlines are replaced by BR
	 *
	 * @returns {String}
	 */
	this.htmlText = function() {
		return self.text.replace(/(?:\r\n|\r|\n|\x8a)/g, '<br>');
	};

}

function TmaTpInfo() {

	// empty default status
	this.satindex			= 0;
	this.flags2				= 0;
	this.polarisation		= 0;
	this.mode				= 0;
	this.modsystem			= 0;
	this.modulation			= 0;
	this.fec				= 0;
	this.pilot				= 0;
	this.channelnumber		= 0;
	this.bandwidth			= 0;
	this.lphpstream			= 0;
	this.frequency			= 0;
	this.symbolrate			= 0;
	this.tsid				= 0;
	this.clocksync			= 0;
	this.networkid			= 0;
	this.originalnetworkid	= 0;

}

function TmaInfBookmarks() {

	// empty default status
	this.startposition	= 0;
	this.count			= 0;

}

