function tmaModule() {

	/**
	 * name of the module. MUST be set in module
	 */
	this.name = "";

	/**
	 * properties of this module
	 * css: string for a single file or array for multiple files
	 * languages: array of supported languages (language-xx.txt)
	 */
	this.manifest = {};

	this.i18 = JK.model({});

	this.templates = {};

	/**
	 * is module initialised?
	 */
	this.inititialised = false;

	/**
	 * is module initialisation in process?
	 */
	this.inititialising = false;

	var settings = null;

	/**
	 * get the modules path
	 */
	this.path = function() {
		return modulesPath + this.name + '/';
	};

	/**
	 * modules setup, loads language, if type: page
	 *
	 * @returns jQueryDeferred
	 */
	this.setup = function() {

		if (this.manifest.type == 'page') {
			return this.loadLanguage();
		} else {
			return $.Deferred().resolve();
		}

	};

	/**
	 * load modules language file
	 * (also loeds manifest, if not existant)
	 *
	 * @returns jQueryDeferred
	 */
	this.loadLanguage = function() {

		var self = this;

		var ctx = tma.fn.getModuleData(this.name);
		return ctx.loadLanguage().done(function(lang) {

			self.i18.extend(lang);

		});


	};

	/**
	 * modules initialisation, loads settings and dependencies (css, js) described in the manifest
	 *
	 * @returns jQueryDeferred
	 */
	this.init = function() {

		var self = this;

		var deferred = $.Deferred();
		var pipe = deferred;

		if (!this.inititialising) {

			if (!this.inititialised) {

				// load language (if it isn't yet)
				pipe = pipe.then(function() {

					return self.loadLanguage();

				});

				// load module settings
				tma.progressMax++;
				pipe = pipe.then(function() {

					return tma.apiListSettings(self.name).done(function(data) {
						settings = data.setting;
					});

				});

				// Get the JavaScript file(s) (deprecated)
				if (this.manifest.js) {
					if (typeof this.manifest.js == 'string') {

						// add script
						tma.progressMax++;
						pipe = pipe.then(function() {

							return tma.fn.loadJs(self.path() + self.manifest.js);

						});

					} else if (typeof this.manifest.js == 'object') {

						// add multiple scripts
						$.each(this.manifest.js, function(i, js) {

							tma.progressMax++;
							pipe = pipe.then(function() {

								return tma.fn.loadJs(self.path() + js);

							});

						});

					}
				}

				// Get the js files from this.dependencies
				if (
						this.dependencies &&
						(typeof this.dependencies.js == 'object')
				) {

					// add multiple scripts
					$.each(this.dependencies.js, function(i, js) {

						tma.progressMax++;
						pipe = pipe.then(function() {

							return tma.fn.loadJs(self.path() + js);

						});

					});

				}

				// Get the css file(s) from manifest (deprecated)
				if (this.manifest.css) {
					if (typeof this.manifest.css == 'string') {

						// add css
						tma.progressMax++;
						pipe = pipe.then(function() {

							return tma.fn.loadCss(self.path() + self.manifest.css);

						});

					} else if (typeof this.manifest.css == 'object') {

						// add multiple css files
						$.each(this.manifest.css, function(i, css) {

							tma.progressMax++;
							pipe = pipe.then(function() {

								return tma.fn.loadCss(self.path() + css);

							});

						});

					}
				}

				// Get the css files from this.dependencies
				if (
						this.dependencies &&
						(typeof this.dependencies.css == 'object')
				) {

					// add multiple css files
					$.each(this.dependencies.css, function(i, css) {

						tma.progressMax++;
						pipe = pipe.then(function() {

							return tma.fn.loadCss(self.path() + css);

						});

					});

				}

				// Get the template file(s) from manifest (deprecated)
				if (this.manifest.templates) {
					if (typeof this.manifest.templates == 'string') {

						// add template
						tma.progressMax++;
						pipe = pipe.then(function() {

							return tma.fn.loadTemplate(self.path() + 'templates/', self.manifest.templates, self, true);

						});

					} else if (typeof this.manifest.templates == 'object') {

						// add multiple templates
						$.each(this.manifest.templates, function(i, template) {

							tma.progressMax++;
							pipe = pipe.then(function() {

								return tma.fn.loadTemplate(self.path() + 'templates/', template, self, true);

							});

						});

					}
				}

				// Get the template files from this.dependencies
				if (
						this.dependencies &&
						(typeof this.dependencies.templates == 'object')
				) {

					// add multiple templates
					$.each(this.dependencies.templates, function(i, template) {

						tma.progressMax++;
						pipe = pipe.then(function() {

							return tma.fn.loadTemplate(self.path() + 'templates/', template, self, false);

						});

					});

				}

				this.inititialising = true;

			}

			deferred.resolve();

		} else {

			// initialising is in process
			deferred.reject();

		}

		return pipe.promise().done(function() {

			// random shot for custom.css
			if (!self.inititialised) {
				tma.fn.loadCss(self.path() + 'custom.css', true);
			}

			self.inititialised = true;
			self.inititialising = false;

		});

	};

	/**
	 * save a modules configuration value
	 *
	 * @param key		name of the configuration parameter
	 * @param value
	 * @returns any value
	 */
	this.setSetting = function (key, value) {

		var self = this;

		settings[key] = value;

		var params = {
				'function':	'setting',
				action:		'set!',
				key:		key,
				value:		value,
				module:		this.name
		};

		return $.getJSON('/api', params).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in " + self.name + ".setSetting()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: '',
					key: key ,
					value: value,
				}
			});

		});

	};

	/**
	 * load a modules configuration value
	 *
	 * @param key		name of the configuration parameter
	 * @param _default	default value, if setting doesn't exist
	 * @returns any value
	 */
	this.setting = function (key, _default) {

		if (
				(!settings) ||
				(settings[key] === undefined) ||
				(settings[key] === null)
		) {
			return _default;
		}

		return settings[key];

	};

	/**
	 * load a local modules configuration value
	 *
	 * @param key		name of the configuration parameter
	 * @param _default	default value, if setting doesn't exist
	 * @returns any value
	 */
	this.localSetting = function (key, _default) {

		return tma.localSetting(key, _default, this.name);

	};

	/**
	 * save a local modules configuration value
	 *
	 * @param key		name of the configuration parameter
	 * @param value
	 */
	this.setLocalSetting = function (key, value) {

		amplify.store( this.name + '_' + key, value );

	};

}
