var TmaReservation = (function() {

	function datePlusDays(date, daysInMs) {

		if ( !(date instanceof Date) ) {
			date = new Date(date);
		}

		// add dist
		if (daysInMs) {
			var hours = date.getHours();
			date = new Date( date.getTime() + daysInMs );
			// DST correction
			if (date.getHours() != hours) {
				date.setHours( hours );
			}
		}

		var result = tma.toPvrDate( date );
		return result;
	}

	function TmaReservation(timer, dist, recurring) {

		// call super class constructor
		TmaReservation._super.constructor.apply(this);

		// empty default data
		this.dist		= dist;
		this.starttime	= datePlusDays( timer.starttime, dist );
		this.endtime	= datePlusDays( timer.endtime, dist );
		this.recurring	= recurring;
		this.slot		= 0;

		this.conflict	= null;
		this.blocking	= null;

		// copy of timer.filename
		this.filename = timer.getFilename();

		// init and apply initial data
		this.init();

		// backward link to timer never changes (not observable for proper destruction)
		this.timer		= timer;

		var self = this;
		var onStarttimeChange = timer.on( 'change', 'modStarttime', function(e, val) {
			self.starttime = datePlusDays(val, self.dist);
		});
		var onEndtimeChange = timer.on( 'change', 'modEndtime', function(e, val) {
			self.endtime = datePlusDays(val, self.dist);
		});
		var onFilenameChange = timer.on( 'change', 'filename', function(e, val) {
			self.filename = val;
		});

		this.destruct = function() {
			this.timer.off( onStarttimeChange );
			this.timer.off( onEndtimeChange );
			this.timer.off( onFilenameChange );

			this.removeConflict();
			this.removeBlocking();
			this.timer = null;
		};

	}

	JK.inherit(TmaReservation, JKobject);

	TmaReservation.prototype.setConflict = function( starttime, endtime ) {

		if ( !this.conflict ) {
			this.conflict = new TmaConflict(this, {
				starttime	: starttime,
				endtime		: endtime,
			});
			tma.timers.conflicts.push( this.conflict );
		} else {
			this.conflict.starttime = starttime;
			this.conflict.endtime = endtime;
		}

	};

	TmaReservation.prototype.removeConflict = function() {

		if ( this.conflict ) {
			var ix = tma.timers.conflicts.indexOf( this.conflict );
			if ( ix >= 0 ) {
				tma.timers.conflicts.splice( ix, 1 );
			}
			this.conflict = null;
		}

	};

	TmaReservation.prototype.setBlocking = function( starttime, endtime, tpIndex1, tpIndex2 ) {

		if ( this.blocking ) {
			if (
					(this.blocking.tpIndex1 == tpIndex1) &&
					(this.blocking.tpIndex2 == tpIndex2)
			) {
				this.blocking.starttime = starttime;
				this.blocking.endtime = endtime;
			} else {
				this.removeBlocking();
			}
		}

		if ( !this.blocking ) {
			this.blocking = new TmaConflict(this, {
				starttime	: starttime,
				endtime		: endtime,
				tpIndex1	: tpIndex1,
				tpIndex2	: tpIndex2,
			});
			for ( var j = 0; j < tma.transponders.length; j++ ) {
				var tp = tma.transponders[j];
				if (
						(tp.transponder_num != tpIndex1) &&
						(tp.transponder_num != tpIndex2)
				) {
					tp.blockings.push( this.blocking );
				}

			}
		}

	};

	TmaReservation.prototype.removeBlocking = function() {

		if ( this.blocking ) {
			for ( var j = 0; j < tma.transponders.length; j++ ) {
				var tp = tma.transponders[j];
				if (
						(tp.transponder_num != this.blocking.tpIndex1) &&
						(tp.transponder_num != this.blocking.tpIndex2)
				) {
					var ix = tp.blockings.indexOf( this.blocking );
					if ( ix >= 0 ) {
						tp.blockings.splice( ix, 1 );
					}
				}

			}
			this.blocking = null;
		}

	};

	TmaReservation.mixin({

		filenameNoExt: function() {
			"watch: {{filename}}";
			return this.filename.replace(/\.[^.]+$/, '');
		},

		starttimeFmt: function() {
			"watch: {{starttime}}";
			return tma.date("H:i", new Date(this.starttime));
		},

		endtimeFmt: function() {
			"watch: {{endtime}}";
			return tma.date("H:i", new Date(this.endtime));
		},

	});

	return TmaReservation;

})();
