var TmaSearchCriteria = (function() {

	// define search criteria object
	function TmaSearchCriteria() {

		// call super class constructor
		TmaSearchCriteria._super.constructor.apply(this);

		this.name = "";
		this.minlen = "";
		this.maxlen = "";
		this.startdate = "";
		this.enddate = "";
		this.starttime = "";
		this.endtime = "";
		this.favgroup = tma.localSetting( 'defaultfavourites', '' );
		this.weekday = {
			1: true,	// monday
			2: true,
			3: true,
			4: true,
			5: true,
			6: true,
			7: true		// sunday
		};
		this.texts = [];
		this.channels = [];

		this.init();

		this.addTextCriteria();

	}
	JK.inherit(TmaSearchCriteria, JKobject);

	/**
	 * add a new text criteria
	 */
	TmaSearchCriteria.prototype.extend = function( criteria ) {
		var i = 0, len = 0;

		// call inherited extend
		this.constructor._super.extend.call(this, criteria, [ 'texts', 'channels' ]);

		// convert text criterias to class TmaSearchText
		if ( criteria.texts ) {
			for ( i = 0, len = criteria.texts.length; i < len; i++ ) {
				this.texts.push( new TmaSearchText(criteria.texts[i]) );
			}
		}

		// convert criteria channels to class TmaChannel
		if ( criteria.channels ) {
			for ( i = 0, len = criteria.channels.length; i < len; i++ ) {
				var channel = tma.channels.channel( criteria.channels[i].logoid );
				if ( channel ) {
					this.channels.push( channel );
				}
			}
		}

	};

	/**
	 * add a new text criteria
	 */
	TmaSearchCriteria.prototype.addTextCriteria = function() {
		this.texts.push( new TmaSearchText() );
	};

	/**
	 * delete a text criteria
	 *
	 * @param {TmaSearchText}	textCriteria
	 */
	TmaSearchCriteria.prototype.deleteTextCriteria = function( textCriteria ) {
		var index = this.texts.findIndex( function(text) {
			return ( text == textCriteria );
		} );
		if ( index >= 0 ) {
			this.texts.splice(index, 1);
		}
	};

	TmaSearchCriteria.prototype.getChannelIds = function() {

		var result = [],
			i = 0,
			len = 0;
		if ( this.channels.length ) {
			// individually selected channels
			for( i = 0, len = this.channels.length; i < len; i++ ) {
				result.push( this.channels[i].logoid );
			}
		} else if ( !this.favgroup ) {
			// all channels
			result.push( "0000000000000000" );
		} else {
			// all channels of a favourite group
			var favChannels = new JKarray();
			tma.channels.getFavChannels(favChannels, this.favgroup);

			$.each(favChannels, function(ix, channel) {
				result.push( channel.logoid );
			});
		}

		return result;

	};

	/**
	 * set all weekdays search permission in one step
	 *
	 * @param {Boolean}	mon		search permission for Monday
	 * ...
	 * @param {Boolean}	sun		search permission for Sunday
	 */
	TmaSearchCriteria.prototype.setWeekdays = function( mon, tue, wed, thu, fri, sat, sun ) {
		this.weekday[1] = mon;
		this.weekday[2] = tue;
		this.weekday[3] = wed;
		this.weekday[4] = thu;
		this.weekday[5] = fri;
		this.weekday[6] = sat;
		this.weekday[7] = sun;
	};

	/**
	 * reset all search options
	 */
	TmaSearchCriteria.prototype.reset = function() {
		this.setWeekdays( true, true, true, true, true, true, true);
		this.name = "";
		this.minlen = "";
		this.maxlen = "";
		this.startdate = "";
		this.enddate = "";
		this.starttime = "";
		this.endtime = "";
		this.favgroup = tma.localSetting( 'defaultfavourites', '' );
		this.channels.length = 0;
		this.texts.length = 0;
		this.addTextCriteria();
	};

	/**
	 * save search criterias
	 *
	 * @param {String}	filename	absolute path on PVRs file system
	 * @return {jQueryDeferred}
	 */
	TmaSearchCriteria.prototype.save = function(filename, searchName) {

		// memorise current name
		var tempSearchName = this.name;
		this.name = searchName;

		/**
		 * this construct allows us to switch to fail(),
		 * if the api returns an error_code <br>
		 * see: return deferred.promise().fail()
		 * http://stackoverflow.com/a/5112734
		 */
		var deferred = $.Deferred();

		//jQuery.post( url [, data ] [, success ] [, dataType ] )
		var url = '/api?function=files&action=store!&filename=' + filename + '&';
		$.post(url, this.stringify(), null, 'json').done( function(data, status, xhr) {

			if (data.error_code !== 0) {
				// switch to fail()
				deferred.reject(
						xhr,
						'tma-error',
						'error_code: ' + data.error_code + '; errno: ' + data.errno
				);
			} else {
				deferred.resolve(data, status, xhr);
			}

		});

		return deferred.promise().fail(function(xhr, errorType, error) {

			// restore original name
			this.name = tempSearchName;
			tma.log({
				title: errorType + " in TmaSearchCriteria.save()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
					parameter: 'fail',
				}
			});

		});


	};

	/**
	 * reset all search options
	 */
	TmaSearchCriteria.prototype.stringify = function() {

		return JSON.stringify( this, function(key, value) {
			switch (key) {
				case 'channels':

					// modify channel list
					var result = [];
					for (var i = 0, len = this.channels.length; i < len; i++ ) {
						result.push({ logoid: this.channels[i].logoid });
					}
					return result;

				default:
					return value;
			}
		}, '\t');

	};

	function makeMJD(inDate) {

		//Create a MJD from JavaScript Date.
		//The JavaScript epoch is based on UTC so it is important NOT to confuse this with local time.

		var tempDate = new Date(0);

		tempDate.setUTCFullYear(inDate.getFullYear());
		tempDate.setUTCMonth(inDate.getMonth());
		tempDate.setUTCDate(inDate.getDate());
		tempDate.setUTCHours(0);
		tempDate.setUTCMinutes(0);
		tempDate.setUTCSeconds(0);

		return 40587 + (tempDate.valueOf() / 86400000);

	}

	/**
	 * creates the search string
	 *
	 * @returns {String}	search string, or error code, if validation fails
	 */
	TmaSearchCriteria.prototype.searchString = function() {

		var errorFlag = 0,
			crlf = "\r\n",
			i = 0,
			len = 0;

		//Loop through each text row to build search string
		var textRowCount = 0;
		var textRowString = "";

		//B;*text*;50
		for ( i = 0, len = this.texts.length; i < len; i++ ) {
			var textCriteria = this.texts[i];

			var err = textCriteria.validate();

			if ( !err ) {
				//Work out the SmartEPG code for the text fields to be searched.
				textRowString += "Search:           ";
				textRowString += textCriteria.location() + ";" + textCriteria.text + ";" + textCriteria.points;
				textRowString += crlf;
				textRowCount++;
			} else {
				errorFlag = err;
			}

		}//END loop through text criteria

		//Assemble the selected channels
		var channelIds = this.getChannelIds();
		var chList = "";
		for( i = 0, len = channelIds.length; i < len; i++ ) {
			chList += "ChannelID:        " + channelIds[i] + crlf;
		}

		// validate selected week days
		if( !this.weekdaysValid() ) {
			errorFlag = 3;
		}

		// validate the duration constraints
		if ( !this.durationsValid() ) {
			errorFlag = 4;
		}

		//Start time and end time default to "0" and "6144" respectively if nothing is specified.
		//If specified, the hours are bit-shifted 8 to the left and then combined with the minutes.

		var start_time = 0;
		var end_time = 6144;

		var temp_min = tma.timeToMinutes(this.starttime);
		if (temp_min !== 0) {
			start_time = Math.floor(temp_min / 60) << 8 | (temp_min % 60);
		}

		temp_min = tma.timeToMinutes(this.endtime);
		if (temp_min !== 0) {
			end_time = Math.floor(temp_min / 60) << 8 | (temp_min % 60);
		}

		// validate Start date and end date
		if ( !this.datesValid() ) {
			errorFlag = 6;
		}

		//*************************************************************
		//** All sanity checks should be completed before this point **
		//*************************************************************

		if ( !errorFlag ) {
			//Create the criteria string for EPGapprentice
			var searchString = "";

			//Because EPGapprentice is designed to read the SmartEPG Searches.txt format, we need to add some fields
			//to trick EPGapprentice into thinking that it has all of the data from Searches.txt.

			//An extension to the SmartEPG format so that EPGapprentice can malloc() its internal search structures.
			searchString += "SearchCount:      " + "1" + crlf; //This MUST be the first field
			searchString += "Name:             " + "WebControl" + crlf;
			//An extension to the SmartEPG format so that EPGapprentice can malloc() its internal regex structures.
			searchString += "TextCount:        " + textRowCount + crlf;
			//An extension to the SmartEPG format so that EPGapprentice can malloc() its internal channel list.
			searchString += "ChannelCount:     " + channelIds.length + crlf;
			searchString += "ShowProgress:     " + "X" + crlf;	//This value does not matter, having the field present is all that is required.
			searchString += "RecordingMode:    " + "0" + crlf;	//This MUST come immediately after the count fields and the name.
			//End trickery.

			searchString += textRowString;

			//List of channelIDs
			searchString += chList;

			//1 = Monday 7 = Sunday
			searchString += "WeekDays:         " + this.weekdays() + crlf;
			// minimum Duration in minutes
			searchString += "MinLength:        " + tma.timeToMinutes(this.minlen) + crlf;
			// maximum Duration in minutes
			searchString += "MaxLength:        " + tma.timeToMinutes(this.maxlen) + crlf;

			searchString += "StartTime:        " + start_time + crlf;
			searchString += "EndTime:          " + end_time + crlf;

			// send the hex representation of the MJD for the required dates.
			searchString += "StartDate:        " + this.startdateMDJ() + crlf;
			searchString += "EndDate:          " + this.enddateMDJ() + crlf;

			searchString += "EndQuery:         " + "1" + crlf;

			return searchString;

		} else {

			/*  1 - No search fields selected
				2 - No search text provided
				3 - No weekdays specified
				4 - Inconsistent lengths
				5 - Inconsistent times
				6 - Inconsistent dates
				7 - Invalid search text */
			return errorFlag;
		}

	};  //END search()

	TmaSearchCriteria.mixin({

		/**
		 * combines the selected weekdays to string for search command
		 *
		 * @returns {String}
		 */
		weekdays: function() {
			"watch: {{weekday.1}} {{weekday.2}} {{weekday.3}} {{weekday.4}} {{weekday.5}} {{weekday.6}} {{weekday.7}}";
			var result = "";
			for( var i = 1; i <= 7; i++ ) {
				if ( this.weekday[i] ) {
					result += i.toString();
				}
			}
			return result;
		},

		/**
		 * check, if any weekday is selected
		 *
		 * @returns {Boolean}
		 */
		weekdaysValid: function() {
			"watch: {{weekday.1}} {{weekday.2}} {{weekday.3}} {{weekday.4}} {{weekday.5}} {{weekday.6}} {{weekday.7}}";

			// weekdays string is valid, if not empty
			return !!this.weekdays();
		},

		/**
		 * validate the min/max length constraints
		 *
		 * @returns {Boolean}
		 */
		durationsValid: function() {
			"watch: {{minlen}} {{maxlen}}";
			//Calculate the length constraints in whole minutes
			var min_len = tma.timeToMinutes(this.minlen);
			var max_len = tma.timeToMinutes(this.maxlen);

			// the length constraints are valid, if
			// * maxlen is empty
			// * or maxlen >= minlen
			return ( (!this.maxlen) || (max_len >= min_len) );
		},

		/**
		 * get MDJ representation of startdate
		 */
		startdateMDJ: function() {
			var d = 0;
			if ( this.startdate ) {
				d = makeMJD( new Date(this.startdate) );
			}
			return ("0000" + d.toString(16)).slice(-4);
		},

		/**
		 * get MDJ representation of enddate
		 */
		enddateMDJ: function() {
			var d = 0;
			if ( this.enddate ) {
				d = makeMJD( new Date(this.enddate) );
			}
			return ("0000" + d.toString(16)).slice(-4);
		},

		/**
		 * validate the min/max date constraints
		 *
		 * @returns {Boolean}
		 */
		datesValid: function() {
			"watch: {{startdate}} {{enddate}}";

			var start_date	= this.startdateMDJ();
			var end_date	= this.enddateMDJ();

			// the date constraints are valid, if
			// * enddate is empty
			// * or enddate >= startdate
			return (
					(!this.enddate) ||
					(end_date >= start_date)
			);
		},

	});

	return TmaSearchCriteria;

})();
