
var TmaStatus = (function() {

	function TmaStatus() {

		// call super class constructor
		TmaStatus._super.constructor.apply(this);

		// empty default status
		this.content			= "";
		this.timestamp			= "";
		this.state				= 0;
		this.substate			= 0;
		this.current_chtype		= 0;
		this.current_chnum		= 0;
		this.current_logoid		= "";
		this.totaldisk			= 0;
		this.freedisk			= 0;
		this.heapsize			= 0;
		this.freeheapsize		= 0;
		this.availheapsize		= 0;

		this.playstatus			= 0;
		this.playmode			= 0;
		this.trickmode			= 0;
		this.speed				= 0;
		this.svctype			= 0;
		this.svcnum				= 0;
		this.logoid				= "";
		this.duration			= 0;
		this.durationsec		= 0;
		this.currentblock		= 0;
		this.totalblock			= 0;

		this.filename			= "";
		this.filename_hex		= "";
		this.absolutefilename	= "";

		/**
		 * recording slots
		 */
		this.recording			= [];

		/**
		 * active recordings
		 */
		this.recordings			= [];

		/**
		 * external drives
		 */
		this.drive_count		= 0;
		this.drives				= [];

		this.online				= false;

		this.activeRecordings	= 0;

		this.blocking			= null;

		/**
		 * totaldisk and freedisk is mapped to drive.size_mb and drive.free_mb
		 * to be compatible with items of drives
		 */
		this.drive				= new TmaStatusDrive();

		// init
		this.init();

		this.on('change', 'current_logoid', function(event, val, obj, prop, oldVal) {
			if (oldVal) {
				var oldChannel = tma.channels.channel(oldVal);
				if (oldChannel) {
					oldChannel.isCurrent = false;
				}
			}
			if (val) {
				var channel = tma.channels.channel(val);
				if (channel) {
					channel.isCurrent = true;
				}
			}
		});

		this.on('change', 'timestamp', function(e, val, status) {
			if (status.playmode === TmaStatus.PLAYMODE_None) {
				//Watching live TV
				var channel = status.channel();
				if (channel) {
					status.currentEvent().load(channel, status.timestamp);
				}
			}
		});

		this.on('change', 'playmode', function(e, val, status) {
			if (status.playmode === TmaStatus.PLAYMODE_None) {
				//Watching live TV
				var channel = status.channel();
				if (channel) {
					status.currentEvent().load(channel, status.timestamp);
				}
			}
		});

		this.on('change', 'absolutefilename', function(event, val, obj, prop, oldVal) {
			var path, file;

			if (oldVal) {
				path = tma.splitPath(oldVal);
				if (
						path &&
						(tma.files.directory == path.directory)
				) {
					// old file is in tma.files folder
					file = tma.files.find( 'name', path.name.replace(/\.inf$/, '') );
					if (file) {
						file.isPlaying = false;
					}
				}
			}

			if (val) {
				path = tma.splitPath(val);
				if (
						path &&
						(tma.files.directory == path.directory)
				) {
					// new file is in tma.files folder
					file = tma.files.find( 'name', path.name.replace(/\.inf$/, '') );
					if (file) {
						file.isPlaying = true;
					}
				}
			}

		});

	}

	JK.inherit(TmaStatus, JKobject);

	TmaStatus.PLAYMODE_None			= 0;		// No playback started
	TmaStatus.PLAYMODE_Playing		= 2;		// Normal Playback
	TmaStatus.PLAYMODE_TempPlaying	= 3;		// Time shifting
	TmaStatus.PLAYMODE_RecPlaying	= 4;		// Time shifting in recording service.
	TmaStatus.PLAYMODE_Mp3			= 5;		// MP3
	TmaStatus.PLAYMODE_Media		= 8;		// Media file

	TmaStatus.prototype.extend = function(status) {

		// extract drive list
		var drives = [];
		if (status.drives) {
			drives = status.drives;
			delete status.drives;
		}

		// call inherited extend
		this.constructor._super.extend.call(this, status);

		// update drives
		var i, j, drive;
		// mark existing drives as "not found"
		for ( j = 0; j < this.drives.length; j++) {
			this.drives[j].found = false;
		}
		// find existing drive
		i = 0;
		while ( i < drives.length ) {
			var d = drives[i];
			var found = false;
			for ( j = 0; j < this.drives.length; j++) {
				drive = this.drives[j];
				if (
						!drive.found &&
						(drive.mountpath == d.mountpath)
				) {
					drive.found = true;
					// update existing drive
					drive.extend(d);
					drives.splice( i, 1 );
					found = true;
					break;
				}
			}

			if (!found) {
				i++;
			}
		}

		// remove disappeared drives
		j = 0;
		while ( j < this.drives.length) {
			drive = this.drives[j];
			if ( !drive.found ) {
				this.drives.splice( j, 1 );
			} else {
				j++;
			}
		}

		// convert each new drive and add it
		for ( i = 0; i < drives.length; i++) {
			drive = new TmaStatusDrive(drives[i]);
			this.drives.add( drive );
		}

	};

	TmaStatus.prototype.currentEvent = function() {
		if (!this.__p.currentEvent) {
			this.__p.currentEvent = new TmaEpgEvent();
			this.__p.currentEvent.setParent( this, 'currentEvent' );
		}
		return this.__p.currentEvent;
	};

	/**
	 * get data of a specific recoding slot by slot number or by logoid
	 *
	 * @param {any} slot_or_logoid		if type is string, search recording by logoid, else by slot number
	 * @returns {TmaStatusRecording}	or false, if there is no data for requested slot
	 */
	TmaStatus.prototype.getRecording = function( slot_or_logoid ) {

		if (typeof slot_or_logoid == 'string') {
			// find logoid
			for (var i = 0; i < this.recording.length; i++) {
				if ( (this.recording[i].rectype) && (this.recording[i].logoid == slot_or_logoid) ) {
					return this.recording[i];
				}
			}
		} else if (this.recording[slot_or_logoid]) {
			// return by slot number
			return this.recording[slot_or_logoid];
		}

		return null;

	};

	/**
	 * get the number of active recordings
	 *
	 * @returns {Number}
	 */
	TmaStatus.prototype.activeRecordingCount = function() {

		var result = 0;
		this.eachRecording(function(slot, recording) {
			result++;
		});
		return result;

	};

	/**
	 * loops through all recordings
	 *
	 * @param {Function}	callback	callback function will be called for each recording with parameters (nr, recording)
	 * @param {Boolean}		activeOnly	optional, default = true, return only active recording slots
	 */
	TmaStatus.prototype.eachRecording = function( callback, activeOnly ) {

		for (var i = 0; i < this.recording.length; i++ ) {
			if ( (activeOnly === false) || (this.recording[i].rectype)) {
				if (callback.apply( this.recording[i], [i, this.recording[i]]) === false) {
					break;
				}
			}
		}
	};

	/**
	 * get data of a specific drive
	 *
	 * @param {Number} nr
	 * @returns {TmaStatusDrive}	or false, if there is no data for requested drive
	 */
	TmaStatus.prototype.drive = function( nr ) {
		if (this.drives[nr]) {
			return this.drives[nr];
		} else {
			return false;
		}
	};

	// handling refresh interval
	var refreshInterval = 0;
	var refreshTimeoutID = null;
	var visiblyInitialised = false;

	/**
	 * set the PVR-status refresh interval
	 *
	 * @param {Number}	secs	0:	stop;
	 * 							>0: start or change interval
	 * @returns {Number} previous value
	 */
	TmaStatus.prototype.setRefreshInterval = function( secs ) {

		if( !visiblyInitialised) {
			var self = this;
			visibly.onVisible(function () {
				console.log('page is visible => restart status update interval (' + refreshInterval + ' secs)');
				self.forceRefresh();
			});

			visibly.onHidden(function () {
				console.log('page is hidden => stop status update interval');
				if (refreshTimeoutID) {
					// refresh timer is running
					clearInterval(refreshTimeoutID);
					refreshTimeoutID = null;
				}
			});
		}

		var oldValue = refreshInterval / 1000;
		refreshInterval = secs * 1000;
		this.forceRefresh();
		return oldValue;

	};

	/**
	 * stops a running refresh timer and starts polling immediatly
	 */
	TmaStatus.prototype.forceRefresh = function() {

		if (refreshTimeoutID) {
			// refresh timer is running
			clearInterval(refreshTimeoutID);
			refreshTimeoutID = null;
		}
		if (refreshInterval) {
			this.load();
			var self = this;
			refreshTimeoutID = setInterval(
					//this.load.bind(this),
					function() {
						self.load();
					},
					refreshInterval
			);
		}

	};

	/**
	 * Get the status object from the PVR
	 * @returns jQueryDeferred
	 */
	TmaStatus.prototype.load = function() {

		var self = this;
		var deferred = $.Deferred();

		var params = {
				'function':	'status',
				action:		'get!'
		};

		$.getJSON('/api', params).done(function(status) {

			self.extend( status );
			self.online = true;

			deferred.resolve(self);

		}).fail(function(xhr, errorType, error) {

			self.online = false;
			deferred.reject(xhr, errorType, error);

		});

		return deferred.promise();

	};

	TmaStatus.mixin({

		/**
		 * get file name of the current playback .inf file (non-UTF Toppy safe)
		 *
		 * @returns {String}
		 */
		getFilename: function() {
			"watch: {{filename}},{{filename_hex}}";
			if (!tma) return '';
			return ( tma.config.utftoppy || !this.filename_hex ) ?
					this.filename || ''
					: tma.decodeHexname(this.filename_hex);
		},

		/**
		 * get file name of the current playback .inf file (non-UTF Toppy safe)
		 *
		 * @returns {String}
		 */
		filenameNoExt: function() {
			"watch: {{filename}},{{filename_hex}}";
			return this.getFilename().replace(/\.[^.]*$/, '');
		},

		channel: function() {
			"watch: {{current_logoid}}";
			if (this.current_logoid) {
				return tma.channels.channel(this.current_logoid);
			} else {
				return null;
			}
		},

		/**
		 * returns the time part of 'timestamp'
		 */
		timestampTime: function() {
			"watch: {{timestamp}}";
			return this.timestamp.slice(-8);
		},

		/**
		 * updates drive object
		 */
		updateDriveSize: function() {
			"watch: {{totaldisk}}";
			this.drive.size_mb = this.totaldisk;
		},

		/**
		 * updates drive object
		 */
		updateDriveFree: function() {
			"watch: {{freedisk}}";
			this.drive.free_mb = this.freedisk;
		},

	});

	TmaStatus.prototype.calcRecordingSlots = function( slots, transponders) {

		function addToTransponders(slot, index) {

			var ti = 0;
			while (ti < transponders.length) {
				if (transponders[ti].index == index) {
					// slot records on existing transponder
					if (transponders[ti].endtime < slot.endtime) {
						transponders[ti].endtime = slot.endtime;
					}
					break;
				}
				ti++;
			}
			if (ti == transponders.length) {
				// new transponder
				transponders.push({
					index: index,
					endtime: slot.endtime,
				});
			}

		}

		var slotNr, channel;
		var recMaxStarttime = '0000/00/00 00:00';
		for (slotNr = 0; slotNr < tma.config.maxrecstreams; slotNr++) {
			/** @returns {TmaStatusRecording} */
			var recording = tma.status.recording[slotNr];
			if ( recording.rectype ) {
				// active recording
				slots[slotNr] = recording;
				if (recording.starttime > recMaxStarttime) {
					recMaxStarttime = recording.starttime;
				}
				channel = recording.channel();
				if (channel) {
					addToTransponders( slots[slotNr], channel.transponderindex );
				}
			}
		}

		// find blocking
		if ( transponders.length == 2 ) {
			var transponder;

			// order transponders by endtime
			transponders.sort( function(a, b) {
				return (a.endtime > b.endtime) ? 1 : (a.endtime == b.endtime) ? 0 : -1;
			});

			// check for blocking transponders
			transponder = transponders[0];
			var endtimeBlock = transponder.endtime;
			var starttimeBlock = recMaxStarttime;
			this.setBlocking( starttimeBlock, endtimeBlock, transponders[0].index, transponders[1].index );

		} else {
			// no blocking
			this.removeBlocking();
		}


	};

	TmaStatus.prototype.setBlocking = function( starttime, endtime, tpIndex1, tpIndex2 ) {

		if ( this.blocking ) {
			if (
					(this.blocking.tpIndex1 == tpIndex1) &&
					(this.blocking.tpIndex2 == tpIndex2)
			) {
				this.blocking.starttime = starttime;
				this.blocking.endtime = endtime;
			} else {
				this.removeBlocking();
			}
		}

		if ( !this.blocking ) {
			this.blocking = new TmaConflict(this, {
				starttime	: starttime,
				endtime		: endtime,
				tpIndex1	: tpIndex1,
				tpIndex2	: tpIndex2,
			});
			for ( var j = 0; j < tma.transponders.length; j++ ) {
				var tp = tma.transponders[j];
				if (
						(tp.transponder_num != tpIndex1) &&
						(tp.transponder_num != tpIndex2)
				) {
					tp.blockings.push( this.blocking );
				}

			}
		}

	};

	TmaStatus.prototype.removeBlocking = function() {

		if ( this.blocking ) {
			for ( var j = 0; j < tma.transponders.length; j++ ) {
				var tp = tma.transponders[j];
				if (
						(tp.transponder_num != this.blocking.tpIndex1) &&
						(tp.transponder_num != this.blocking.tpIndex2)
				) {
					var ix = tp.blockings.indexOf( this.blocking );
					if ( ix >= 0 ) {
						tp.blockings.splice( ix, 1 );
					}
				}

			}
			this.blocking = null;
		}

	};

	return TmaStatus;

})();
