var TmaStatusRecording = (function() {

	function TmaStatusRecording(recording) {

		// call super class constructor
		TmaStatusRecording._super.constructor.apply(this);

		// empty default recording
		this.slot			= 0;
		this.rectype		= 0;
		this.tuner			= 0;
		this.svctype		= 0;
		this.svcnum			= 0;
		this.logoid			= "";
		this.duration		= 0;
		this.starttime		= "";
		this.endtime		= "";
		this.recordedsec	= 0;

		this.filename		= "";
		this.filename_hex	= "";

		/**
		 * dummy ID type to be compatible with TmaTimer
		 */
		this.id = -1;

		/**
		 * dummy Reservation type to be compatible with TmaTimer
		 *
		 * @type {Number} allways 0: One Time
		 */
		this.reservationtype = 0;

		/**
		 * data for recording modifications
		 */
		this.modified = false;
		this.modEndtime = '';

		// init
		this.init(recording);

		this.on('change', 'logoid', function(event, val, obj, prop, oldVal) {
			var ix = 0;
			if (oldVal) {
				var oldChannel = tma.channels.channel(oldVal);
				if (oldChannel) {
					ix = oldChannel.recordings.indexOf(this);
					if (ix >= 0) {
						oldChannel.recordings.splice(ix, 1);
					}
					if (tma.status.activeRecordings) {
						tma.status.activeRecordings--;
					}
					ix = tma.status.recordings.indexOf(this);
					if (ix >= 0) {
						tma.status.recordings.splice(ix, 1);
					}
				}
			}
			if (val) {
				var channel = tma.channels.channel(val);
				if (channel) {
					tma.status.activeRecordings++;
					channel.recordings.add( this );
					ix = tma.status.recordings.indexOf(this);
					if (ix == -1) {
						tma.status.recordings.push(this);
					}
				}
			}
		});

		this.on('change', 'filename_hex', function(event, val, obj, prop, oldVal) {
			var file;

			if (oldVal) {
				if (tma.files.directory == '/mnt/hd/DataFiles') {
					// old file is in tma.files folder
					file = tma.files.find( 'name_hex', oldVal );
					if (file) {
						file.recording = null;
					}
				}
			}

			if (val) {
				if (tma.files.directory == '/mnt/hd/DataFiles') {
					// new file is in tma.files folder
					file = tma.files.find( 'name_hex', val );
					if (file) {
						file.recording = obj;
					}
				}
			}

		});

	}

	JK.inherit(TmaStatusRecording, JKobject);

	TmaStatusRecording.mixin({

		/**
		 * get file name of this recording slot (non-UTF Toppy safe)
		 *
		 * @param {Boolean]	removeExt	remove file extension? (default = false)
		 * @returns {String}
		 */
		getFilename: function(removeExt) {
			"watch: {{filename}}";

			/** @returns {String} */
			var name = (tma.config.utftoppy || !this.filename_hex) ?
					this.filename
					: tma.decodeHexname(this.filename_hex);

			if (removeExt) {
				return name.replace(/\.[^.]+$/, '');
			} else {
				return name;
			}

		},

		filenameNoExt: function() {
			"watch: {{filename}}";
			return this.getFilename( true );
		},

		channel: function() {
			"watch: {{logoid}}";
			return tma.channels.channel(this.logoid);
		},

		durationFmt: function() {
			"watch: {{duration}}";
			return tma.formatDuration(this.duration);
		},

		starttimeFmt: function() {
			"watch: {{starttime}}";
			return tma.date("H:i", new Date(this.starttime));
		},

		endtimeFmt: function() {
			"watch: {{endtime}}";
			return tma.date("H:i", new Date(this.endtime));
		},

		modEndtimeFmt: function() {
			"watch: {{modEndtime}}";
			return tma.date("H:i", new Date(this.modEndtime));
		},

		recordedFmt: function() {
			"watch: {{recordedsec}}";
			return tma.formatDuration( parseInt(this.recordedsec / 60) );
		},

		/**
		 * reset modifications to original values
		 */
		modReset: function() {
			this.modEndtime = this.endtime;
		},

		/**
		 * update modEndtime on endtime change
		 * and recalc timer conflicts
		 */
		onEndtimeChange: function() {
			"watch: {{endtime}}";
			this.modEndtime = this.endtime;
			tma.timers.calcSlots();
		},

		/**
		 * recalc timer conflicts on rectype change
		 */
		onRectypeChange: function() {
			"watch: {{rectype}}";
			tma.timers.calcSlots();
		},

		/**
		 * set modified on endtime/modEndtime change
		 */
		setModified: function() {
			"watch: {{endtime}},{{modEndtime}}";
			this.modified = (this.modEndtime != this.endtime);
		},

	});

	return TmaStatusRecording;

})();
