/**
 * encapsulates running tap-list object of json response
 *
 * @param {TmaTaps}	taps
 */
var TmaTaps = (function() {

	function TmaTaps(taps) {

		// call super class constructor
		TmaTaps._super.constructor.apply(this);

		// empty default data
		this.count		= 0;
		this.timestamp	= "";
		this.content	= "taps";
		this.taps		= [];

		// init
		this.init( taps );

		this.taps.on('add', '*', function(e, tap) {
			if (tma.files.pathSection() == 'ProgramFiles') {
				for (var j = 0; j < tma.files.files.length; j++ ) {
					var file = tma.files.files[j];
					var path = tma.files.directory + '/' + file.name;
					if (tap.file_name == path) {
						file.isRunningTap = true;
						break;
					}
				}
			}
		});

		this.taps.on('del', '*', function(e, nil, parent, index, tap) {
			if (tma.files.pathSection() == 'ProgramFiles') {
				for (var j = 0; j < tma.files.files.length; j++ ) {
					var file = tma.files.files[j];
					var path = tma.files.directory + '/' + file.name;
					if (tap.file_name == path) {
						file.isRunningTap = false;
						break;
					}
				}
			}
		});

	}

	JK.inherit(TmaTaps, JKobject);

	TmaTaps.prototype.extend = function(taps) {

		// extract taplist
		var taplist = [];
		if (taps.tap) {
			taplist = taps.tap;
			delete taps.tap;
		}

		// call inherited extend
		this.constructor._super.extend.call(this, taps);

		// update taplist
		var i, j, tap;
		// mark existing taps as "not found"
		for ( j = 0; j < this.taps.length; j++) {
			this.taps[j].found = false;
		}
		// find existing tap
		i = 0;
		while ( i < taplist.length ) {
			var t = taplist[i];
			var found = false;
			for ( j = 0; j < this.taps.length; j++) {
				tap = this.taps[j];
				if (
						!tap.found &&
						(tap.tapid == t.tapid) &&
						(tap.file_name == t.file_name)
				) {
					tap.found = true;
					// update existing tap
					tap.extend(t);
					taplist.splice( i, 1 );
					found = true;
					break;
				}
			}

			if (!found) {
				i++;
			}
		}

		// remove disappeared tap
		j = 0;
		while ( j < this.taps.length) {
			tap = this.taps[j];
			if ( !tap.found ) {
				this.taps.splice( j, 1 );
			} else {
				j++;
			}
		}

		// convert each new tap and add it
		for ( i = 0; i < taplist.length; i++) {
			tap = new TmaTap(taplist[i]);
			this.taps.add( tap );
		}

	};


	/**
	 * loops through all taps
	 *
	 * @param {Function}	callback	callback function will be called for each tap with
	 * 									parameters ({Number} tapIndex, {TmaTap} tap)
	 */
	TmaTaps.prototype.eachTap = function( callback ) {

		for (var i = 0; i < this.taps.length; i++ ) {
			if (callback.apply( this.taps[i], [i, this.taps[i]]) === false) {
				break;
			}
		}
	};

	/**
	 * loops through all taps
	 *
	 * @param {Number}	tapid
	 * @param {String}	fullpath	optional, also compares the filepath of the TAP
	 * @returns {TmaTap}	or false
	 */
	TmaTaps.prototype.find = function( tapid, fullpath ) {

		var result = false;

		/** @param {TmaTap} tap */
		this.eachTap(function(i, tap) {

			if (tap.tapid == tapid) {
				if (
						(!fullpath) ||
						(fullpath == tap.file_name)
				) {
					result = tap;
					return false; // break
				}
			}
		});

		return result;

	};

	return TmaTaps;

})();
