/**
 * encapsulates timer-list object of json response
 *
 * @param {TmaTimers}	timers
 */
var gTestCount = 0;
var TmaTimers = (function() {

	// sort timers by starttime
	function timerCompare(a, b)  {
		if (a.starttime < b.starttime) return -1;
		if (a.starttime > b.starttime) return 1;
		return 0;
	}

	function TmaTimers() {

		// call super class constructor
		TmaTimers._super.constructor.apply(this);

		// empty default data
		this.count		= 0;
		this.timestamp	= '';
		this.content	= 'timers';
		this.timers		= [];

		/**
		 * all reservations
		 */
		this.reservations = [];

		/**
		 * collected conflicts
		 */
		this.conflicts = [];

		// init
		this.init();

		this.timers.sort(timerCompare);
		this.reservations.sortBy('starttime');

		/**
		 * synchronise each channel's timer list
		 */
		var self = this;

		this.timers.on('add', '*', function(e, timer ) {
			var channel = timer.channel();
			if ( channel ) {
				channel.timers.add( timer );
			}
		});

		this.timers.on('del', '*', function(e, nil, parent, index, delTimer ) {
			var channel = delTimer.channel();
			if ( channel ) {
				var ix = channel.timers.indexOf( delTimer );
				if ( ix >= 0 ) {
					channel.timers.splice( ix, 1 );
				}
			}
			//delTimer.destruct();
		});

		this.reservations.on('add', '*', function() {
			self.calcSlots();
		});

		this.reservations.on('del', '*', function() {
			self.calcSlots();
		});

	}

	JK.inherit(TmaTimers, JKobject);

	TmaTimers.prototype.extend = function(timers) {

		// extract timerlist
		var timerlist = [];
		if (timers.timers) {
			timerlist = timers.timers;
			delete timers.timers;
		}

		// call inherited extend
		this.constructor._super.extend.call(this, timers);

		// update timerlist
		var i, j, timer;
		// mark existing timers as "not found"
		for ( j = 0; j < this.timers.length; j++) {
			this.timers[j].found = false;
		}
		// find existing timer
		i = 0;
		while ( i < timerlist.length ) {
			var t = timerlist[i];
			t.timestamp = this.timestamp;
			var found = false;
			for ( j = 0; j < this.timers.length; j++) {
				timer = this.timers[j];
				if (
						!timer.found &&
						(timer.logoid == t.logoid) &&
						(timer.starttime == t.starttime) &&
						(timer.endtime == t.endtime)
				) {
					timer.found = true;
					// update existing timer (timestamp)
					timer.extend(t);
					timerlist.splice( i, 1 );
					found = true;
					break;
				}
			}

			if (!found) {
				i++;
			}
		}

		// remove disappeared timer
		j = 0;
		while ( j < this.timers.length) {
			timer = this.timers[j];
			if ( !timer.found ) {
				this.timers.splice( j, 1 );
			} else {
				j++;
			}
		}

		// convert each new timer and add it
		for ( i = 0; i < timerlist.length; i++) {
			timer = new TmaTimer(timerlist[i]);
			timer.id = timer.id || this.getUniqueId();
			this.timers.add( timer );
		}

	};

	/**
	 * Get a timers listing from the PVR
	 *
	 * @returns {jQueryDeferred}
	 */
	TmaTimers.prototype.load = function() {

		var deferred = $.Deferred();

		var params = {
				'function':	'timer',
				action:		'get!',
		};

		var self = this;
		$.getJSON('/api', params, function(timers) {

			self.extend(timers);

			deferred.resolve(self);

		}).fail(function(xhr, errorType, error) {

			tma.log({
				title: errorType + " in TmaTimers.load()",
				details: {
					error: error,
					xhrReadyState: xhr.readyState,
					xhrStatus: xhr.status,
				}
			});

			deferred.reject(xhr, errorType, error);

		});

		return deferred.promise();

	};

	/**
	 * find timer by unique id
	 *
	 * @param {Number}	timerid
	 * @returns {TmaTimer}	or false
	 */
	TmaTimers.prototype.find = function( timerid ) {

		for (var i = 0; i < this.timers.length; i++ ) {

			if (this.timers[i].id == timerid) {
				return this.timers[i];
			}
		}

		return false;
	};

	var calcSlotsId = null;

	function calcSlots( self ) {
		gTestCount++;

		var transponders = [];

		function addToTransponders(slot, index) {

			var ti = 0;
			while (ti < transponders.length) {
				if (transponders[ti].index == index) {
					// slot records on existing transponder
					if (transponders[ti].endtime < slot.endtime) {
						transponders[ti].endtime = slot.endtime;
					}
					break;
				}
				ti++;
			}
			if (ti == transponders.length) {
				// new transponder
				transponders.push({
					index: index,
					endtime: slot.endtime,
				});
			}

		}

		function slotSort(a, b) {
			return (slots[a].endtime > slots[b].endtime) ? 1 : (slots[a].endtime == slots[b].endtime) ? 0 : -1;
		}

		var slots = [ null, null, null, null ];

		tma.status.calcRecordingSlots( slots, transponders );

		var slotNr, channel;
		var maxtime = tma.toPvrDate( (new Date(tma.status.timestamp)).getTime() + 7 * 24*60*60000 );
		var reservation;
		for( var i = 0; i < self.reservations.length; i++) {
			var gotConflict = false;
			var gotBlocking = false;
			/** @returns {TmaReservation} */
			reservation = self.reservations[i];
			if (reservation.starttime > maxtime) break;
			transponders.length = 0;
			var slotFound = false;
			slotNr = 0;
			while ( !slotFound || (slotNr < slots.length) ) {
				if (
						(slotNr == slots.length) ||
						!slots[slotNr] ||
						(slots[slotNr].endtime <= reservation.starttime)
				) {
					// slot is available
					if (!slotFound) {
						slots[slotNr] = reservation;
						reservation.slot = slotNr;
						slotFound = true;
					} else {
						if ( slotNr >= tma.config.maxrecstreams) {
							slots.splice(slotNr, 1);
						} else {
							slots[slotNr] = null;
							slotNr++;
						}
						continue;
					}
				}

				var slot = slots[slotNr];
				if (slot) {
					channel = (slot.timer) ? slot.timer.channel() : slot.channel();
					if (channel) {
						addToTransponders( slots[slotNr], channel.transponderindex );
					}
				}

				slotNr++;
			}
			var endtimeC1, endtimeC2;
			if ( slots.length > tma.config.maxrecstreams ) {
				// conflict: to much recordings
				gotConflict = true;

				var tempSlots = [];
				for( slotNr = 0; slotNr < slots.length; slotNr++ ) {
					tempSlots[slotNr] = slotNr;
				}

				// order tempSlots by endtime
				tempSlots.sort( slotSort );

				// find conflict end
				while (tempSlots.length > tma.config.maxrecstreams) {
					var tempSlot = slots[ tempSlots.shift() ];
					endtimeC1 = tempSlot.endtime;
				}

			}
			if (transponders.length > 1) {
				var transponder;

				// order transponders by endtime
				transponders.sort( function(a, b) {
					return (a.endtime > b.endtime) ? 1 : (a.endtime == b.endtime) ? 0 : -1;
				});

				if (transponders.length > 2) {
					// conflict: no free tuner available
					gotConflict = true;

					// find conflict end
					while (transponders.length > 2) {
						transponder = transponders.shift();
						endtimeC2 = transponder.endtime;
					}

				}

				// check for blocking transponders
				if (transponders.length == 2) {
					gotBlocking = true;
					transponder = transponders[0];
					var endtimeBlock = transponder.endtime;
					var starttimeBlock = reservation.starttime;
					reservation.setBlocking( starttimeBlock, endtimeBlock, transponders[0].index, transponders[1].index );
				}

			}

			if (gotConflict) {

				var endtime;
				if ( !endtimeC1 ||
					(endtimeC2 && (endtimeC2 > endtimeC1) )
				) {
					endtime = endtimeC2;
				} else {
					endtime = endtimeC1;
				}
				if (endtime > reservation.endtime) {
					endtime = reservation.endtime;
				}

				reservation.setConflict(reservation.starttime, endtime);

			} else {

				reservation.removeConflict();

			}
			if ( !gotBlocking ) {

				reservation.removeBlocking();

			}
		}

		calcSlotsId = null;
	}

	TmaTimers.prototype.calcSlots = function( ) {

		if (!calcSlotsId) {
			calcSlotsId = setImmediate(calcSlots, this);
		}

	};

	/**
	 * static counter value for unique IDs
	 * @see getUniqueId()
	 */
	var nextId = 0;

	/**
	 * returns a unique timer ID
	 *
	 * @returns {Number}
	 */
	TmaTimers.prototype.getUniqueId = function() {
		return ++nextId;
	};

	return TmaTimers;

})();
