/**
 * Channel seletion utility utility module
 */
tma.registerModule( channelselect );
channelselect.prototype = new tmaModule();
function channelselect() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'channelselect';

	var self = this;

	this.dependencies = {
			css: [
			      "channelselect.css"
			],
			templates: [
				"content.html"
			]
	};

	this.templateData = JK.model({
		channels: new JKarray(),
		channel: null,
	});

	this.templateData.on('change', 'channel', function(e, val, obj, prop, oldVal) {
		if (oldVal instanceof TmaChannel) {
			oldVal.extendProp('chselIsCurrent', false);
		}
		if (val instanceof TmaChannel) {
			val.extendProp('chselIsCurrent', true);
		}
	});

	this.blacklist = [];

	/**
	 * prepare and return data for template
	 *
	 *  @param {JKobject}	channelParent
	 *  @param {Array}		blacklist
	 */
	this.getTemplateData = function( channelParent, blacklist ) {

		this.blacklist = blacklist || [];

		if ( !TmaChannel.prototype.chselPrev ) {
			// add prev/next to TmaChannel
			TmaChannel.mixin({

				chselPrev: function() {
					var index = self.templateData.channels.indexOf(this);
					if (index == -1) {
						return null;
					}
					if (index === 0) {
						// this is the first channel in the list => previous one is the last
						index = self.templateData.channels.length;
					}
					return self.templateData.channels[index - 1];
				},

				chselNext: function() {
					var index = self.templateData.channels.indexOf(this);
					if (index == -1) {
						return null;
					}
					if (index === self.templateData.channels.length - 1) {
						// this is the last channel in the list => next one is the first
						index = -1;
					}
					return self.templateData.channels[index + 1];
				},

			});
		}

		// prepare channels list
		var group = tma.channels.fav_name;
		this.templateData.channel = tma.channels.channel( tma.status.current_logoid );
		if (this.templateData.channel) {
			if ( tma.channels.favChannels.indexOf(this.templateData.channel) == -1) {
				// current channel does not exist in current fav_group
				group = '';
			}
		}
		tma.channels.getFavChannels(
				this.templateData.channels,
				group,
				this.blacklistFilter
		);

		return {
			i18g:		tma.i18,
			channels:	this.templateData.channels,
			current:	channelParent,
			favgroups:	tma.favourites.groups,
		};

	};

	this.blacklistFilter = function( channel ) {

		// return true, if logoid is not in blacklist
		return ( self.blacklist.indexOf(channel.logoid) == -1 );

	};

	function scrollInView() {

		var $ul = $('.chsel UL');
		var $li = $ul.find('LI[is-current="true"]');
		if ($li.length) {
			$ul.scrollTop(0);
			var pos = $li.position();
			var ulHeight = $ul.height();
			var liHeight = $li.height();

			if (
					( pos.top > ulHeight - liHeight ) ||
					( pos.top < 0 )
			) {
				$ul.scrollTop( pos.top - parseInt((ulHeight - liHeight) / 2) );
			}
		}

	}

	function setCurrentChannelsFavGroup(  ) {

		var selectedid = $('.chsel LI[is-current="true"]').attr('channelid');
		var channel = tma.channels.channel( selectedid );
		if (channel) {

			if ( self.templateData.channels.indexOf(channel) == -1) {
				// show all channels
				$('.chsel-dropdown SELECT[name="favourites"]').val('').change();
			}

		}

	}

	this.addEventHandler = function( $template, onSelect ) {

		$template.on('change', '.chsel-dropdown SELECT[name="favourites"]', function() {

			//tma.channels.fav_name = $(this).val();
			tma.channels.getFavChannels(
					self.templateData.channels,
					$(this).val(),
					self.blacklistFilter
			);

			// scroll current channel in view
			scrollInView();

		});

		// open/close list
		$template.on('click', '.chsel-current', function() {

			var $dropdown = $('.chsel-dropdown-frame');
			$dropdown.slideToggle();
			if ($dropdown.is(':visible') ) {

				setCurrentChannelsFavGroup();

				// scroll current channel in view
				scrollInView();

			}
		});

		// select a channel
		$template.on('click', '.chsel-channel[channelid]', function() {
			if ( onSelect ) {
				var channelid = $(this).attr('channelid');
				self.templateData.channel = tma.channels.channel( channelid );
				onSelect( channelid );
			}
			$('.chsel-dropdown-frame').hide();
		});

	};

}
