/**
 * Column Configuration utility utility module
 */
tma.registerModule( columnconfig );
columnconfig.prototype = new tmaModule();
function columnconfig() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'columnconfig';

	var self = this;

	this.dependencies = {
			css: [
			      "columnscfg.css"
			],
			templates: [
				"content.html"
			]
	};

	var configState;

	/**
	 * returns data required by columns config element
	 *
	 * @param {JKmodel}		currentCfg		current config; [ {name: {string}, display: {bool} }, ... ]
	 * @param {Object}		columnCaptions	type: {clmClass: string caption, ... }
	 * @returns {Object}
	 */
	this.getTemplateData = function(currentCfg, columnCaptions) {

		configState = currentCfg;

		return {
			cc_i18: this.i18,
			columns: configState,
			clmCaption: function() { return columnCaptions[this.name]; },
		};

	};

	/**
	 * add event handler of config element to a given template
	 *
	 * @param {jQueryObject}	$template
	 * @param {String}			targetSelector	selector of the table, that is configured
	 * @param {Function}		onCfgChange		callback: function(newColumnsCfg)
	 */
	this.addEventHandler = function( $template, targetSelector, onCfgChange ) {

		function triggerCfgChange() {

			if (onCfgChange) {
				var cells = {};
				configState.forEach(function(clm) {
					cells[clm.name] = clm.display;
				});

				onCfgChange(cells);
			}

		}

		$template.on('click', 'BUTTON.clmcfg-shift-right', function() {

			var colName = $(this).parents('TD').attr('clm-name');
			var index = configState.findIndex( function(clm) {
				return (clm.name == colName);
			} );

			if (index >= 0 && index < configState.length - 1) {

				configState.splice( index, 2, configState[index+1], configState[index]);

				triggerCfgChange();

				$( targetSelector ).find(
						('TD[clm-name="{0}"],TH[clm-name="{0}"]').format( colName )
				).each(function() {
					$(this).insertAfter( $(this).next() );
				});
			}

		});

		$template.on('click', 'BUTTON.clmcfg-shift-left', function() {

			var colName = $(this).parents('TD').attr('clm-name');
			var index = configState.findIndex( function(clm) {
				return (clm.name == colName);
			} );

			if (index > 0 && index < configState.length) {

				configState.splice( index - 1, 2, configState[index], configState[index-1]);

				triggerCfgChange();

				$( targetSelector ).find(
						('TD[clm-name="{0}"],TH[clm-name="{0}"]').format( colName )
				).each(function() {
					$(this).insertBefore( $(this).prev() );
				});
			}
		});

		$template.on('click', 'INPUT.clmcfg-display', function() {

			var colName = $(this).parents('TD').attr('clm-name');

			configState.forEach(function(clm) {
				if (clm.name == colName) {
					clm.display = !clm.display;
					return false; // break
				}
			});

			triggerCfgChange();

			$( targetSelector ).find( ( 'TD[clm-name="{0}"],TH[clm-name="{0}"]' ).format( colName ) ).toggle();

		});

	};

}
