/**
 * config module
 */
tma.registerModule( configpage );
configpage.prototype = new tmaModule();
function configpage() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'configpage';

	this.pageGroup = 'config';

	var installation = false;

	var self = this;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		// set font size from setting
		var fs = parseFloat( tma.localSetting('fontsize', $('BODY').css('fontSize')) );
		$('BODY').css('fontSize', fs + 'px');

		if (tma.fn.newModules.length) {
			installation = true;
			this.installNewModules();
		}

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			$('#config').addClass('globals-config');
			if (!installation) {
				self.insertConfig();
				$('#config').show();
			}

			$('#timestamp').hide();

		});

	};

	/**
	 * this page gets removed, so remove config class
	 */
	this.cleanup = function() {

		$('#config').removeClass('globals-config');
		$('#timestamp').show();

	};

	/**
	 * displays a popup with a module description
	 *
	 * @param {String}	modulename
	 */
	this.showModuleInfo = function(modulename) {

		var moduleData = tma.fn.getModuleData(modulename);
		if (
				(moduleData) &&
				(tma.i18.modules) &&
				(tma.i18.modules[modulename]) &&
				(tma.i18.modules[modulename].module.info)
		) {
			var i18 = tma.i18.modules[modulename];

			this.getModuleDescription(moduleData).done(function(info) {

				var infoWindow = new jkqWindow();
				infoWindow.width = 450;

				infoWindow.showModal();
				infoWindow.setTitle( i18.module.title );
				infoWindow.closeOnOutsideClick();

				infoWindow.container.find('DIV.jkqWindowContent')
				.append(
						'<div class="module-info">' +
						info +
						'</div>'
				);

				infoWindow.adjustPos();

			});

		}

	};

	this.insertConfig = function () {

		// all available modules, that have a manifest
		var availableModules = tma.fn.getModules();

		// all used modules
		var usedModules = [];
		$.each(tma.modulesToLoad(), function( i, name ) {
			var moduleData = tma.fn.getModuleData(name);
			if (moduleData) {
				usedModules.push(moduleData);
			}
		});

		$('DIV#config').append( this.templates.config.render(
				{	// data
					i18: this.i18,
					cfg: tma.config,
					availableModules: availableModules,
					usedModules: usedModules,
				},
				{	// partials
					module: this.templates.module
				}
		) );

		/* modules config (device - all modules) */

		$.each(usedModules, function( i, moduleData ) {
			$('.available-modules .module[module="' + moduleData.name + '"]').hide();
		});

		// load missing languages
		/** @param {tmaModuleData} moduleData */
		$.each(availableModules, function( i, moduleData ) {

			if (moduleData.manifest.type != 'util') {
				moduleData.loadLanguage().done(function(i18) {
					var $mds = $('.module[module="' + moduleData.name + '"]');
					$mds.find('SPAN').text( (i18.module || {}).title || '');
					$mds.attr('data-type', moduleData.manifest.type);
					if (i18.module && i18.module.info) {
						$mds.find('BUTTON.ic-info').show();
					}
				});
			}

		});

		// set current startpage
		var startpage = tma.localSetting('startpage', 'none');
		$('.used-modules .modules-list .module[module="' + startpage + '"]').addClass('start-module');

		$('.modules-list').on('click', 'DIV.module', function(e) {

			// click on div.module works like a <select> list with multiple selections
			if (!e.metaKey && !e.ctrlKey) {

				$('.modules-list .module-selected')
				.removeClass('module-selected')
				.find('INPUT').prop('checked', false);

			}

			$(this).find('INPUT').click();

		});

		$('.used-modules .modules-list').on('click', '.module INPUT', function(e) {

			// click on used module deselects all available modules
			$('.available-modules .module-selected').removeClass('module-selected');
			$('.available-modules INPUT:checked').prop('checked', false);

		});

		$('.available-modules .modules-list').on('click', '.module INPUT', function(e) {

			// click on available module deselects all used modules
			$('.used-modules .module-selected').removeClass('module-selected');
			$('.used-modules INPUT:checked').prop('checked', false);

		});

		$('.modules-list').on('click', '.module INPUT', function(e) {

			e.stopPropagation();

			// a module was (de-)selected
			$(this).parents('.module').toggleClass('module-selected');

			var isUsedModule = $(this).parents('.modules-frame').hasClass('used-modules');
			var $sm = $(this).parents('.modules-list').find('.module-selected');

			// set buttons visibility
			var $buttons = $('DIV#config DIV.module-buttons');
			$buttons.find('BUTTON.remove-module'	).attr( 'disabled', ($sm.length === 0) || !isUsedModule );
			$buttons.find('BUTTON.install-module'	).attr( 'disabled', ($sm.length === 0) || isUsedModule );
			$buttons.find('BUTTON.level-up'			).attr( 'disabled', ($sm.length === 0) || !isUsedModule || $sm.first().is(':first-child') );
			$buttons.find('BUTTON.level-down'		).attr( 'disabled', ($sm.length === 0) || !isUsedModule || $sm.last().is(':last-child') );
			$buttons.find('BUTTON.start-module'	).attr( 'disabled', ($sm.length !=  1) || !isUsedModule || !$sm.first().is('[data-type="page"]') );

		});

		// remove button
		$('.module-buttons BUTTON.remove-module').click(function() {

			// get all selected modules in 'used modules'
			var $selected = $('.used-modules .module-selected');

			// show them in 'available modules'
			$selected.each(function() {

				var name = $(this).attr('module');
				$('.available-modules .module[module="' + name + '"]').show();

			});

			$selected.remove();

			// update button's state
			$(this).siblings('.level-up'		).attr( 'disabled', true );
			$(this).siblings('.level-down'		).attr( 'disabled', true );
			$(this).siblings('.remove-module'	).attr( 'disabled', true );
			$(this).siblings('.install-module'	).attr( 'disabled', true );

		});

		// add button
		$('.module-buttons .install-module').click(function() {

			// get all selected modules in 'available modules'
			var $selectedAvail = $('.available-modules .module-selected');

			// add them to 'used modules'
			$selectedAvail.each(function() {

				$('.used-modules .modules-list').append( $(this ).clone());

			});

			$selectedAvail.removeClass('module-selected').hide().find('INPUT').attr('checked', false);

			// update button's state
			var $selectedUsed = $('.used-modules .module-selected');
			$(this).siblings('.level-up'		).attr( 'disabled', $selectedUsed.first().is( ':first-child' ) );
			$(this).siblings('.level-down'		).attr( 'disabled', $selectedUsed.last().is( ':last-child' ) );
			$(this).siblings('.remove-module'	).attr( 'disabled', false );
			$(this).siblings('.install-module'	).attr( 'disabled', true );


		});

		// up button
		$('.module-buttons BUTTON.level-up').click(function() {

			// get all selected modules
			var $selected = $('.used-modules .modules-list .module-selected');

			// move them one positiop up
			$selected.each(function() {

				$(this).prev().before( $(this) );

			});

			// update button's state
			$(this).attr( 'disabled', $selected.first().is( ':first-child' ) );
			$(this).siblings('.level-down').attr( 'disabled', false );

		});

		// down button
		$('.module-buttons BUTTON.level-down').click(function() {

			// get all selected modules
			var $selected = $('.used-modules .modules-list .module-selected');

			// move them one positiop down
			$selected.each(function() {

				$(this).next().after( $(this) );

			});

			// update button's state
			$(this).attr( 'disabled', $selected.last().is( ':last-child' ) );
			$(this).siblings('.level-up').attr( 'disabled', false );

		});

		// home button
		$('.module-buttons BUTTON.start-module').click(function() {

			// get first selected page module
			var $selected = $('.used-modules .modules-list .module-selected[data-type="page"]').first();

			if ( $selected.length ) {

				if ( $selected.hasClass('start-module') ) {
					$selected.removeClass('start-module');
				} else {

					// remove previous home
					$('.used-modules .modules-list .start-module').removeClass('start-module');
					// set home
					$selected.addClass('start-module');

				}

			}

		});

		// save button
		$('.module-buttons BUTTON.save-module-config').click(function() {

			// get all 'used modules'
			var $selected = $('.used-modules .modules-list .module');
			// collect and save used module names
			var usedList = [];
			$selected.each(function() {
				usedList.push( $(this).attr('module') );
			});

			tma.setLocalSetting( 'modules', usedList.join(',') );

			// start page
			var home = $('.used-modules .modules-list .start-module').attr('module');
			if (home) {
				tma.setLocalSetting( 'startpage', home );
			} else {
				tma.setLocalSetting( 'startpage', null );
			}

			window.location.hash = '';
			window.location.reload();

		});

		// reset button
		$('.module-buttons BUTTON.reset-modules').click(function() {

			new jkqDialog({
				title: self.i18.reset_confirm_caption,
				text: self.i18.reset_confirm_text,
				buttons: {
					ok:				 tma.i18.respond_yes,
					jkqDialogCancel: tma.i18.respond_cancel
				}
			}).confirm(function(button) {

				if (button == 'ok') {

					location.replace('reset-modules.html');

				}

			});
		});

		// info button
		$('.modules-list BUTTON.ic-info').click(function(e) {

			e.stopPropagation();

			var modulename = $(this).parents('.module').attr('module');
			self.showModuleInfo(modulename);

		});

		$('DIV#config SELECT[name="duration-format"]')		//DMC format duration
		.val( tma.localSetting('durationformat', 0) )
		.change(function() {

			tma.setLocalSetting( 'durationformat', $(this).val() );

		});


		/* logo aspect ratio setting (PVR - all modules) */

		if (tma.logos) {

			// change logos aspect ratio setting
			$('DIV#config SELECT[name="logo-aspect-ratio"]')
			.val( tma.setting('logodisplaywidth', 54) )
			.change(function() {

				tma.setSetting( 'logodisplaywidth', $(this).val() );
				tma.refreshLogoStyles();

			});

		} else {

			// if logos aren't available, hide logos aspect ratio setting
			$('DIV#config DIV.logos-aspect-ratio').hide();

		}

		/* font-size setting (device - all modules) */

		$('DIV#config DIV.fontsize .less').click(function() {

			var fs = parseInt( tma.localSetting('fontsize', $('BODY').css('fontSize')) );

			if (fs > 5) {
				fs--;
				$('BODY').css('fontSize', fs + 'px');
				tma.setLocalSetting('fontsize', fs);
				$(window).resize();
				$('#body').css('margin-top', $('#head').outerHeight());
			}

		});
		$('DIV#config DIV.fontsize .more').click(function() {

			var fs = parseFloat( tma.localSetting('fontsize', $('BODY').css('fontSize')) );

			fs++;
			$('BODY').css('fontSize', fs + 'px');
			tma.setLocalSetting('fontsize', fs);
			$(window).resize();
			$('#body').css('margin-top', $('#head').outerHeight());

		});

		/* language mode setting (local - all modules) */

		// add languages
		if (this.i18.languages) {
			var languageIds = Object.keys(this.i18.languages);
			for (var i = 0; i < languageIds.length; i++) {
				var languageId = languageIds[i];

				$('DIV#config SELECT[name="language-mode"]').append(
						'<option value="' + languageId + '">' + this.i18.languages[languageId] + ' (' + languageId + ')</option>'
				);

			}
		}

		// save language button
		$('DIV#config DIV.language-mode .button-save').click(function() {

			tma.setLocalSetting( 'languagemode', $('DIV#config SELECT[name="language-mode"]').val() );

			window.location.reload();

		});

		/* display PVR name setting (local - all modules) */

		$('DIV#config INPUT[name="show-pvr-name"]')
		.click(function() {

			tma.setLocalSetting( 'showpvrname', $(this).is(':checked') );
			$('#MainMenu').attr('show-name', $(this).is(':checked'));
			$(window).resize();

		})
		.prop( 'checked', tma.localSetting('showpvrname', true) )
		;

		// reset confirm dialogs
		$('DIV#config DIV.reset-confirm-dialogs BUTTON').click(function() {

			var key;
			var data = amplify.store();
			for ( key in data ) {
				if (key.indexOf('jkq-dialog-autoskip_') === 0) {
					$('DIV#config DIV.reset-confirm-dialogs BUTTON').removeAttr("disabled");
					amplify.store(key, null);
				}
			}
			$('DIV#config DIV.reset-confirm-dialogs BUTTON').attr("disabled", true);

		});

		// apply current setting
		$('DIV#config SELECT[name="language-mode"]')
		.val( tma.localSetting('languagemode', 'pvr') );

		var key;
		var data = amplify.store();
		for ( key in data ) {
			if (key.indexOf('jkq-dialog-autoskip_') === 0) {
				// there is at least one dialog to reset
				$('.reset-confirm-dialogs BUTTON').removeAttr("disabled");
				break;
			}
		}

	};

	var descriptionsCache = {};

	/**
	 * reads the module description
	 *
	 * @param {tmaModuleData} moduleData
	 */
	this.getModuleDescription = function( moduleData ) {

		var deferred = $.Deferred();
		var pipe = deferred;

		if (
				(moduleData) &&
				(tma.i18.modules) &&
				(tma.i18.modules[moduleData.name])
		) {
			var i18 = tma.i18.modules[moduleData.name];

			if ( i18.module) {

				if ( ( i18.module.info || '').match( /\.html$/ ) ) {

					// moduleData.module.i18.module.info is a filename
					var filename = moduleData.path() + 'lng/' + i18.module.info;

					if (!descriptionsCache[moduleData.name]) {

						// load module description
						$.get(filename).done(function(data) {

							descriptionsCache[moduleData.name] = data;
							deferred.resolve(descriptionsCache[moduleData.name]);

						});

					} else {

						deferred.resolve(descriptionsCache[moduleData.name]);

					}

				} else {

					// moduleData.module.i18.module.info is a text string
					descriptionsCache[moduleData.name] = i18.module.info || '';
					deferred.resolve(descriptionsCache[moduleData.name]);

				}
			}

		} else {

			deferred.resolve('');

		}

		return pipe.promise();

	};

	this.installNewModules = function() {

		this.init().done(function() {

			var popup = new jkqWindow();

			popup.showModal();
			popup.setTitle( self.i18.install.title ); //
			$(popup).on('close', function() {
				installation = false;
				if ( $('#config').hasClass('globals-config') ) {
					self.insertConfig();
					$('#config').show();
				}
			});

			var $content = popup.container.find('DIV.jkqWindowContent');

			$content.append( self.templates.install.render(
					{	// data
						i18: self.i18,
						newModules: tma.fn.newModules
					},
					{	// partials
						module: self.templates.module
					}
			) );

			// load missing languages
			/** @param {tmaModuleData} moduleData */
			$.each(tma.fn.newModules, function( i, moduleData ) {

				moduleData.loadLanguage().done(function(i18) {
					var $mds = $content.find('.module[module="' + moduleData.name + '"]');
					$mds.find('SPAN').text( (i18.module || {}).title || '');
					$mds.attr('data-type', moduleData.manifest.type);
					if (i18.module && i18.module.info) {
						$mds.find('BUTTON.ic-info').show();
					}
				});

			});

			// info button
			$content.on('click', 'BUTTON.ic-info', function(e) {

				var modulename = $(this).parents('DIV.module').attr('module');
				self.showModuleInfo(modulename);

			});

			// (de-)select module by checkbox
			$content.on('click', 'DIV.module INPUT', function(e) {

				$(this).parents('DIV.module').toggleClass('module-selected');

				// en-/disable install button
				$content.find('BUTTON[fn="install"]').prop("disabled", $content.find('INPUT:checked').length === 0);

			});

			// (de-)select module by click on name
			$content.on('click', 'DIV.module SPAN', function(e) {

				$(this).parents('DIV.module').find('INPUT').click();

			});

			// button close
			$content.on('click', 'BUTTON[fn="close"]', function(e) {

				popup.close();

			});

			// button install
			$content.on('click', 'BUTTON[fn="install"]', function(e) {

				var usedModules = tma.modulesToLoad();
				$content.find('INPUT:checked').each(function() {
					var modulename = $(this).parents('DIV.module').attr('module');
					usedModules.push(modulename);
				});
				tma.setLocalSetting( 'modules', usedModules.join(',') );

				window.location.reload();

			});

			popup.adjustPos();

		});

	};

}
