/**
 * Directory selection modul
 */
tma.registerModule( dirselector );
dirselector.prototype = new tmaModule();
function dirselector() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'dirselector';

	var self = this;

	var popup = new jkqWindow();
	popup.width = 300;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'selectDirectory' );

		return $.Deferred().resolve();

	};

	/**
	 * show directory tree in a window
	 *
	 * @param {tmaFeature}	f
	 * @param {String}		root		tree root
	 * @param {String}		current		current folder
	 * @param {Function}	onSelect	callback function(path), that is called when a directory is selected
	 */
	this.selectDirectory = function(f, root, current, onSelect) {

		/**
		 * since this contains asynchronous calls, pause the chain
		 * and decide with the result of the call, whether to stop,
		 * or to continue
		 */
		f.pause();

		this.init().done(function() {

			popup.showModal();
			popup.setTitle( self.i18.window_caption );

			/** @param {TmaFiles} folders */
			tma.apiGetFolders(root).done(function(folder) {

				popup.container.find('DIV.jkqWindowContent')
				.html( self.templates.content.render(
						// data
						folder,
						{	// partials
							folder: self.templates.folder,
						}
				) )

				.on('click', '.dir-ctx', function(e) {
					e.stopPropagation();

					var $folder = $(this);
					$('.dir-ctx.selected').removeClass('selected');
					$folder.addClass('selected');

					if ( $folder.attr('subdirs') ) {
						$folder.find('.dir-sub').slideToggle(function() {
							$folder.toggleClass('expanded');
						});
					} else {

						var dir = $folder.attr('path');
						self.insertFolder($folder, dir);

					}

				})

				.on('dblclick', '.dir-ctx', function(e) {
					e.stopPropagation();

					var $selected = $(this);
					if (onSelect) {
						onSelect( $selected.attr('path') );
					}
					popup.close();

				});

				popup.container.find('DIV.jkqWindowFooter')
				.html( self.templates.footer.render() )
				.on('click', '.dir-ok', function() {

					var $selected = $('.dir-ctx.selected');
					if (onSelect && $selected.length) {
						onSelect( $selected.attr('path') );
					}
					popup.close();

				});

				popup.adjustPos();

				// open current
				var path = current.replace(/^\/mnt\//, '').split('/');
				var deferred = $.Deferred();
				var pipe = deferred;

				$.each(path, function(i, folder) {

					pipe = pipe.then(function() {

						var dir = '/mnt/' + path.slice(0, i + 1).join('/');
						var $folder = popup.container.find('.dir-ctx[path="' + dir + '"]');

						return self.insertFolder($folder, dir);

					});

				});

				deferred.resolve();

				// set current as selected
				pipe.then(function() {

					var $folder = popup.container.find('.dir-ctx[path="' + current + '"]');
					$folder.addClass('selected');

					// scroll in view
					var $frame = popup.container.find('.dir-selector');
					var pFrame = $frame.offset();
					var pFolder = $folder.offset();
					if (pFolder && pFrame) {
						var hFrame = $frame.innerHeight();
						var top = pFolder.top - pFrame.top;
						if (top > hFrame - 21) {
							$frame.animate({scrollTop: '+=' + top}, 200);
						} else if (top < 0) {
							$frame.animate({scrollTop: '+=' + (top - hFrame + 21)}, 200);
						}
					}

				});

			}).fail(function(xhr, errorType, error) {

				popup.container.find('DIV.jkqWindowContent').html(
						tma.i18.error.ajax.format(errorType, error)
				);

			});

			popup.container.find('DIV.jkqWindowContent').html('<img src="css/img/wait.gif">');
			popup.adjustPos();

			/**
			 * this method handled the feature, so
			 * stop the feature chain
			 * and return this popup instance
			 */
			f.stop(popup);

		});

	};

	this.insertFolder = function($folder, path) {

		var deferred = $.Deferred();

		tma.apiGetFolders(path).done(function(files) {

			$folder.find('.dir-sub').append(
					self.templates.folder.render(files)
			).slideToggle(function() {
				$folder.addClass('expanded');
				deferred.resolve();
			});
			$folder.attr('subdirs', files.files.length);

		});

		return deferred.promise();

	};

}
