/**
 * EPG module
 */
tma.registerModule( epgnow );
epgnow.prototype = new tmaModule();
function epgnow() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'epgnow';

	this.pageGroup = 'epg';

	var self = this;

	this.dependencies = {

			css: [
			      "epgnow.css"
			      ],

			js: [
			     "../genrecolours/genrecolours.js"
			     ],

			templates: [
					     "content.html",
					     "config.html",
					     "head.html",
			]
	};

	var $contentTmpl, $configTmpl, $stickyTmpl;
	var timeHandler, favHandler;

	/**
	 * initialize module
	 */
	this.setup = function() {

		// register tmaFeatures
		tma.fn.addFeature( this, 'appendMenuTab' );

		return this.loadLanguage();

	};

	/**
	 * append tab to menu
	 *
	 * @param {tmaFeature}	f
	 */
	this.appendMenuTab = function( f ) {

		tma.call('addPage', ['MainMenu', this]);

	};

	/**
	 * Display the channels obtained from the PVR in the browser window
	 */
	this.show = function() {

		this.init().done(function() {

			// select channels list
			tma.channels.fav_name = tma.prefs.favGroup;

			// load genre list
			tma.apiEpgGenre().done(function() {

				self.showSticky();
				self.showEPG();

				tma.call('addConfigItem', ['MainMenu', self]);

				self.insertConfig();

			});

		});

	};

	/**
	 * this page gets removed
	 */
	this.cleanup = function() {

		// remove event handling
		$(window).on('scroll.epgnow');

		tma.status.off(timeHandler);
		tma.channels.off(favHandler);
		timeHandler = null;
		favHandler = null;

		// remove templates
		JK.removeTemplate( $contentTmpl );
		JK.removeTemplate( $configTmpl );
		JK.removeTemplate( $stickyTmpl );

	};

	/**
	 * callback function of tma.status.on( 'change', 'timestamp' ... )
	 * call it directly without parameters, to update immediately
	 */
	var updateEpgEvents = function(e, val) {

		var wh = window.innerHeight;
		$('TR[channelid]').each(function() {

			// test, if element is in viewport
			var rect = this.getBoundingClientRect();
			if (rect.top < wh && rect.bottom > 0) {

				// update event
				var channel = tma.channels.channel( $(this).attr('channelid') );
				channel.epgnowTimestamp = val || tma.status.timestamp;
				channel.epgnowPage = prefs.page;

			}
		});
	};

	this.showSticky = function() {

		// render template
		$stickyTmpl = JK.template(
				this.templates.head,
				{
					i18:			this.i18,
					prefs:			prefs,
				}
		).appendTo( '.sticky-content' );

		$stickyTmpl.on('click', '.epgnow-now', function() {
			prefs.page = 0;
			updateEpgEvents();
			// additional deferred update, if rows moved in view due to height change
			setTimeout(updateEpgEvents, 100);
		});

		$stickyTmpl.on('click', '.epgnow-next', function() {
			prefs.page++;
			updateEpgEvents();
			// additional deferred update, if rows moved in view due to height change
			setTimeout(updateEpgEvents, 100);
		});

		$stickyTmpl.on('click', '.epgnow-prev', function() {
			prefs.page--;
			updateEpgEvents();
			// additional deferred update, if rows moved in view due to height change
			setTimeout(updateEpgEvents, 100);
		});

	};

	/**
	 * load and display an EPG data
	 */
	this.showEPG = function () {

		// extend channel objects
		for (var i = 0; i < tma.channels.channels.length; i++) {
			var channel = tma.channels.channels[i];

			// create internal event object, if not existant
			if ( !channel.epgnowEvent ) {
				channel.extend({
					epgnowEvent: null,
					epgnowTimestamp: '',
					epgnowPage: 0,
				});
			}
		}

		// extend TmaChannel class
		if (!TmaChannel.prototype.epgnowUpdate) {
			TmaChannel.mixin({
				epgnowUpdate: function() {
					"watch: {{epgnowTimestamp}}, {{epgnowPage}}";

					/**
					 * timestamp or epgnow-pagenumber has changed,
					 * so calculate a new epgnowEvent for this channel
					 */

					var channel = this;

					function setNowEvent(event) {

						channel.epgnowEvent = event;
						// set formats
						channel.epgnowEvent.extend({
							fmtDate		: tma.date(self.i18.shortdate_fmt, event.starttime_local),
						});

					}

					// make sure, epg data ist available for current timestamp/page
					this.getEvents( this.epgnowTimestamp ).done(function(events) {
						if (events.length) {

							var index = channel.epgEvents.indexOf(events[0]);
							index += channel.epgnowPage;
							if ( index < channel.epgEvents.length) {
								setNowEvent( channel.epgEvents[index] );
							} else {
								// we need later EPG events
								var timestamp = channel.epgEvents[ channel.epgEvents.length - 1 ].endtime_local;
								channel.getEvents( timestamp ).done(function(events) {
									if ( index < channel.epgEvents.length) {
										setNowEvent( channel.epgEvents[index] );
									}
								});
							}
						}
					});

				},
			});
		}

		// render template
		$contentTmpl = JK.template(
				this.templates.content,
				{
					i18:			this.i18,
					prefs:			prefs,
					channels:		tma.channels.favChannels,
				}
		).appendTo( '#content' );

		if (!timeHandler) {
			timeHandler = tma.status.on( 'change', 'timestamp', updateEpgEvents);
		}

		if (!favHandler) {
			favHandler = tma.channels.on('change', 'fav_name', function() {
				// deferred update
				setTimeout(updateEpgEvents, 100);
			});
		}

		$(window).on('scroll.epgnow', function() {
			updateEpgEvents();
		});

		updateEpgEvents();

		// update timers
		tma.timers.load();

		this.createEventHandler();

	};

	this.createEventHandler = function() {

		// show timer form
		$contentTmpl.on('click', '.epgnow-evt-reservation', function(e) {

			// prevent opening EPG details
			e.stopPropagation();

			var timer = tma.timers.find( $(this).attr('timerid') );
			if (timer) {

				// hope, that any module handles the 'editTimer' event
				tma.call( 'editTimer', [ timer ] );

			}

		});

		// show event details
		$contentTmpl.on('click', '.epgnow-channel', function(e) {

			var $event = $(this);
			var channel = tma.channels.channel( $(this).attr('channelid') );
			if ( channel ) {

				var eventid = parseInt( $(this).attr('eventid') );
				if ( eventid ) {

					var event = channel.epgEvents.find( function(event) {
						return event.eventid == eventid;
					});

					if ( event ) {

						// check, if any module handles the 'showEvent' event
						// and prevent reopening of the same info
						if (
								(tma.fn.showEvent) &&
								!$event.hasClass('jk-mark')
						) {

							tma.fn.showEvent( event ).done(function(popup) {
								popup.$relatedEvent = $event;
								$(popup).on('close', function() {
									this.$relatedEvent.removeClass('jk-mark');
								});
							});

							$event.addClass('jk-mark');

						}

					}
				}
			}

		});

	};

	this.insertConfig = function () {

		$configTmpl = JK.template(
				this.templates.config,
				{
					i18:			this.i18.config,
					i18g:			tma.i18,	// global language def.
					prefs:			prefs,
					gPrefs:			tma.prefs,
					favgroups:		tma.favourites.groups,
				}
		).appendTo('DIV#config');

		/* favourite group */
		var $fav = $('DIV#config SELECT[name="favourites"]');
		$fav.val( tma.prefs.favGroup );
		$fav.change(function() {

			tma.prefs.setFavGroup( $(this).val() );
			tma.channels.fav_name = $(this).val();

		});

		/* description */
		$configTmpl.find('INPUT[name="description"]').click(function() {

			prefs.setDescrMode( $(this).val() );

		})
		// select
		.filter('[value="' +  prefs.descrMode + '"]' ).prop('checked', true);

		/* text */
		$configTmpl.find('INPUT[name="text"]').click(function() {

			prefs.setTextMode( $(this).val() );

		})
		// select
		.filter('[value="' +  prefs.textMode + '"]' ).prop('checked', true);

		/* text depending on description */
		$configTmpl.find('INPUT[name="textdep"]').click(function() {

			prefs.setTextDepMode( $(this).prop('checked') );

		})
		// select
		.prop( 'checked', prefs.textDepMode );

		/* show/hide genre */
		$configTmpl.find('INPUT[name="genre"]').click(function() {

			prefs.setGenreMode( $(this).val() );

		})
		// select
		.filter('[value="' +  prefs.genreMode + '"]' ).prop('checked', true);

		tma.fn.getModule('genrecolours').createConfig(
				prefs.genreColoring,
				function(isEnabled) { // onEnableChange
					prefs.setGenreColoring(isEnabled);
				}
		).appendTo('DIV#config');

	};

	/* Settings */

	var prefs = JK.model({

			/**
			 * defines, which event is displayed.
			 * 0: current event
			 * 1: later(1)
			 * ...
			 */
			page: 0,

			/**
			 * description display mode (none|first|*full)
			 *
			 * @type {String}
			 */
			descrMode: self.localSetting('descr', 'full'),

			/**
			 * set the description display mode (none|first|full)
			 *
			 * @param {String}	set
			 */
			setDescrMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'first') ||
						(set == 'full')
				) {
					self.setLocalSetting( 'descr', set );
					this.descrMode = set;
				}
			},

			/**
			 * text display mode (none|*first|full)
			 *
			 * @return {String}
			 */
			textMode: self.localSetting('text', 'first'),

			/**
			 * set the text display mode (none|first|full)
			 *
			 * @param {String}	set
			 */
			setTextMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'first') ||
						(set == 'full')
				) {
					self.setLocalSetting( 'text', set );
					this.textMode = set;
				}
			},

			/**
			 * text-if-no-description display mode (true|*false)
			 *
			 * @type {Boolean}
			 */
			textDepMode: self.localSetting('textdep', false),

			/**
			 * set the text-if-no-description-mode (true|false)
			 *
			 * @param {Boolean}	set
			 */
			setTextDepMode: function(set) {
				if (set !== undefined) {
					self.setLocalSetting( 'textdep', set === true );
					this.textDepMode = set;
				}
			},

			/**
			 * genre display mode (none|*show)
			 *
			 * @return {String}
			 */
			genreMode: self.localSetting('genre', 'show'),

			/**
			 * set the genre display mode (none|show)
			 *
			 * @param {String} set
			 */
			setGenreMode: function(set) {
				if (
						(set == 'none') ||
						(set == 'show')
				) {
					self.setLocalSetting( 'genre', set );
					this.genreMode = set;
				}
			},

			/**
			 * genre-coloring (*on|off)
			 *
			 * @return {Boolean}
			 */
			genreColoring: parseInt( self.localSetting( 'genre-coloring', 1 ) ) == 1,

			/**
			 * set the genre-coloring (on/off)
			 *
			 * @param {Boolean}	set		new genre-coloring setting
			 */
			setGenreColoring: function(set) {
				self.setLocalSetting( 'genre-coloring', (set) ? 1 : 0 );
				this.genreColoring = (set) ? true : false;
			},

	});

}
