/**
 * multi channel select utility module
 */
tma.registerModule( multichselect );
multichselect.prototype = new tmaModule();
function multichselect() {

	/**
	 * IMPORTANT: the modules name; equal to directory and script file name
	 */
	this.name = 'multichselect';

	var self = this;

	this.dependencies = {
			css: [
			      "multichselect.css"
			],
			templates: [
				"content.html"
			]
	};

	/**
	 * init and show selector
	 *
	 * @param {Object}	target		Object with properties
	 * 								favgroup: selected favourites group
	 * 								channels: list of selected channels
	 */
	this.showSelector = function(target) {

		this.init().done(function() {

			self.createPopup(target);

		});
	};

	/**
	 * create window
	 *
	 * @param {Object}	target		Object with properties
	 * 								favgroup: selected favourites group
	 * 								channels: list of selected channels
	 * @returns {jkqWindow}
	 */
	this.createPopup = function(target) {

		var popup = new jkqWindow();
		popup.width = 350;

		popup.show();
		popup.setTitle( self.i18.heading );
		popup.closeOnOutsideClick();

		var $content = popup.container.find('DIV.jkqWindowContent');

		var favChannels = new JKarray();
		tma.channels.getFavChannels(favChannels, target.favgroup);

		var $selector = JK.template(
				this.templates.content,
				{
					i18g:		tma.i18,
					channels:	favChannels,
					favgroups:	tma.favourites.groups,
				}
		).appendTo( $content )


		.on('change', 'SELECT[name="favourites"]', function() {

			var favgroup = $(this).val();
			if ( favgroup != target.favgroup ) {
				tma.channels.getFavChannels(
						favChannels,
						favgroup
				);
				target.favgroup = favgroup;
				target.channels.length = 0;
			}
		})

		// toggle selection
		.on('click', 'LI', function() {
			$(this).toggleClass('selected');
			var channelid = $(this).attr('channelid');
			var index = target.channels.findIndex( function(channel) {
				return channel.logoid == channelid;
			});
			if ( index == -1 ) {
				// new channel
				target.channels.push( tma.channels.channel( channelid ) );
			} else {
				// remove channel
				target.channels.splice(index, 1);
			}
		})

		;

		// mark selected channels
		for( var i = 0, len = target.channels.length; i < len; i++ ) {
			$selector.find('LI[channelid="' + target.channels[i].logoid + '"]').addClass( 'selected' );
		}

		// set current favourites group
		$selector.find('SELECT[name="favourites"]').val(target.favgroup).change();

		popup.adjustPos({
			hAlign:	'center',
			effect:	'slide-rtl'
		});

		$(popup).on( 'close', function() {
			JK.removeTemplate( $selector );
		} );

		return popup;

	};

}
